@@ -5,12 +5,6 @@ use strict;
 use POSIX;
 use Locale::PO;
 
-my $version = '0.0.0';
-if ($ARGV[0] =~ /^-v(.+)/) {
-    $version = $1;
-    shift @ARGV;
-}
-
 my $pot_creation_date = strftime "%Y-%m-%d %H:%M:%S +0000", gmtime();
 
 use integer;
@@ -41,7 +35,7 @@ print << "END";
 #
 msgid ""
 msgstr ""
-"Project-Id-Version: survex-$version\\n"
+"Project-Id-Version: survex\\n"
 "Report-Msgid-Bugs-To: olly\@survex.com\\n"
 "POT-Creation-Date: $pot_creation_date\\n"
 "PO-Revision-Date: YEAR-MO-DA HO:MI:SE +ZONE\\n"
@@ -51,10 +45,14 @@ msgstr ""
 "Content-Transfer-Encoding: 8bit\\n"
 END
 
-my $num_list = Locale::PO->load_file_asarray("po_codes");
+my $num_list = Locale::PO->load_file_asarray("survex.pot");
 my $first = 1;
 foreach my $po_entry (@{$num_list}) {
-    my $msgno = $po_entry->dequote($po_entry->msgstr);
+    my $msgno = '';
+    my $ref = $po_entry->reference;
+    if (defined $ref && $ref =~ /^n:(\d+)$/m) {
+	$msgno = $1;
+    }
     if ($first) {
 	$first = 0;
 	next if ($po_entry->msgid eq '""');
@@ -64,7 +62,7 @@ foreach my $po_entry (@{$num_list}) {
 	$msg = $msgs{$msgno};
 	delete $msgs{$msgno};
     } else {
-	print STDERR "Message number $msgno is in po_codes but not found in source - preserving\n" unless $po_entry->obsolete;
+	print STDERR "Message number $msgno is in survex.pot but not found in source - preserving\n" unless $po_entry->obsolete;
 	$msg = $po_entry->dequote($po_entry->msgid);
     }
     if (defined $po_entry->automatic) {
@@ -77,6 +75,7 @@ foreach my $po_entry (@{$num_list}) {
 	for (@{$uses[$msgno]}) {
 	    print "\n#: ", $_;
 	}
+	print "\n#: n:$msgno";
     }
     print "\n#, c-format" if $msg =~ /\%[a-z0-9]/;
     if ($msg =~ s/(?:^|[^\\])"/\\"/g) {
@@ -86,7 +85,7 @@ foreach my $po_entry (@{$num_list}) {
     print "#~ " if $po_entry->obsolete;
     print "msgid \"$msg\"\n";
     print "#~ " if $po_entry->obsolete;
-    print "msgstr \"$msgno\"\n";
+    print "msgstr \"\"\n";
 }
 
 for my $msgno (sort keys %msgs) {
@@ -96,10 +95,11 @@ for my $msgno (sort keys %msgs) {
 	print "\n#: ", $_;
     }
     my $msg = $msgs{$msgno};
+    print "\n#: n:$msgno";
     print "\n#, c-format" if $msg =~ /\%[a-z0-9]/;
     if ($msg =~ s/(?:^|[^\\])"/\\"/g) {
 	print STDERR "Escaping unescaped \" in message number $msgno\n";
     }
     print "\nmsgid \"$msg\"\n";
-    print "msgstr \"$msgno\"\n";
+    print "msgstr \"\"\n";
 }
