@@ -4,7 +4,7 @@
 //  Main class for Aven.
 //
 //  Copyright (C) 2001 Mark R. Shinwell.
-//  Copyright (C) 2002,2003,2004,2005,2006,2011 Olly Betts
+//  Copyright (C) 2002,2003,2004,2005,2006,2011,2013 Olly Betts
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -31,9 +31,11 @@
 
 #include "cmdline.h"
 #include "message.h"
+#include "useful.h"
 
 #include <assert.h>
 #include <signal.h>
+#include <stdio.h>
 
 #include <wx/confbase.h>
 #include <wx/image.h>
@@ -88,23 +90,27 @@ bool Aven::Initialize(int& my_argc, wxChar **my_argv)
 {
     // wxWidgets passes us wxChars, which may be wide characters but cmdline
     // wants UTF-8 so we need to convert.
+#ifdef __GNUC__
+    // This is a GCC extension.
     char *utf8_argv[my_argc + 1];
+#else
+    vector<char *> utf8_argv(my_argc + 1);
+#endif
     for (int i = 0; i < my_argc; ++i){
 	utf8_argv[i] = strdup(wxString(my_argv[i]).mb_str());
     }
     utf8_argv[my_argc] = NULL;
 
-    msg_init(utf8_argv);
+    msg_init(&utf8_argv[0]);
     select_charset(CHARSET_UTF8);
     /* Want --version and decent --help output, which cmdline does for us.
      * wxCmdLine is much less good.
      */
     cmdline_set_syntax_message(/*[SURVEY_FILE]*/269, 0, NULL);
-    cmdline_init(my_argc, utf8_argv, short_opts, long_opts, NULL, help, 0, 1);
+    cmdline_init(my_argc, &utf8_argv[0], short_opts, long_opts, NULL, help, 0, 1);
     getopt_first_response = cmdline_getopt();
     return wxApp::Initialize(my_argc, my_argv);
 }
-#define real_argv argv
 #else
 static char ** real_argv;
 
@@ -164,7 +170,11 @@ bool Aven::OnInit()
 	wxString msg_lang_str(msg_lang, wxConvUTF8);
 	const char *lang = msg_lang2 ? msg_lang2 : msg_lang;
 	wxString lang_str(lang, wxConvUTF8);
+#if wxCHECK_VERSION(2,9,0)
+	loc->Init(msg_lang_str, lang_str, msg_lang_str);
+#else
 	loc->Init(msg_lang_str, lang_str, msg_lang_str, true, true);
+#endif
 	// The existence of the wxLocale object is enough - no need to keep a
 	// pointer to it!
     }
@@ -172,6 +182,10 @@ bool Aven::OnInit()
     wxString survey;
     bool print_and_exit = false;
 
+#ifdef __WXMSW__
+    char ** real_argv = argv;
+#endif
+
     while (true) {
 	int opt;
 	if (getopt_first_response) {
@@ -198,7 +212,7 @@ bool Aven::OnInit()
     if (real_argv[optind]) {
 	fnm = wxString(real_argv[optind], wxConvUTF8);
 	if (fnm.empty() && *(real_argv[optind])) {
-	    wxGetApp().ReportError(wxT("File argument's filename has bad encoding"));
+	    ReportError(wxT("File argument's filename has bad encoding"));
 	    return false;
 	}
     }
@@ -228,7 +242,7 @@ bool Aven::OnInit()
     bool maximized = (width == -1);
     bool full_screen = (width <= -2);
     if (width <= 0 || height <= 0) {
-#if wxUSE_DISPLAY // wxDisplay was added in wx 2.5
+#if wxUSE_DISPLAY
 	wxRect geom = wxDisplay().GetGeometry();
 	pos.x = geom.x;
 	pos.y = geom.y;
@@ -321,6 +335,10 @@ Aven::SetPageSetupDialogData(const wxPageSetupDialogData & psdd)
 
 void Aven::ReportError(const wxString& msg)
 {
+    if (!m_Frame) {
+	wxMessageBox(msg, APP_NAME, wxOK | wxICON_ERROR);
+	return;
+    }
     AvenAllowOnTop ontop(m_Frame);
     wxMessageDialog dlg(m_Frame, msg, APP_NAME, wxOK | wxICON_ERROR);
     dlg.ShowModal();
@@ -329,11 +347,7 @@ void Aven::ReportError(const wxString& msg)
 wxString
 wmsg(int msg_no) 
 {
-#if !wxCHECK_VERSION(2,8,0)
-    return wxString(msg(msg_no), wxConvUTF8);
-#else
     return wxString::FromUTF8(msg(msg_no));
-#endif
 }
 
 const wxString &
@@ -349,18 +363,31 @@ wmsg_cfgpth()
 extern "C" void
 aven_v_report(int severity, const char *fnm, int line, int en, va_list ap)
 {
-   wxString m;
-   if (fnm) {
-       m = wxString(fnm, wxConvUTF8);
-       if (line) m += wxString::Format(wxT(":%d"), line);
-       m += wxT(": ");
-   }
+    wxString m;
+    if (fnm) {
+	m = wxString(fnm, wxConvUTF8);
+	if (line) m += wxString::Format(wxT(":%d"), line);
+	m += wxT(": ");
+    }
 
-   if (severity == 0) {
-       m += wmsg(/*warning*/4);
-       m += wxT(": ");
-   }
+    if (severity == 0) {
+	m += wmsg(/*warning*/4);
+	m += wxT(": ");
+    }
 
-   m += wxString::FormatV(wmsg(en), ap);
-   wxGetApp().ReportError(m);
+    char buf[1024];
+    vsnprintf(buf, sizeof(buf), msg(en), ap);
+    m += wxString(buf, wxConvUTF8);
+    if (wxTheApp == NULL) {
+	// We haven't initialised the Aven app object yet.
+	if (!wxInitialize()) {
+	    fputs(buf, stderr);
+	    PUTC('\n', stderr);
+	    exit(1);
+	}
+	wxMessageBox(m, APP_NAME, wxOK | wxICON_ERROR);
+	wxUninitialize();
+    } else {
+	wxGetApp().ReportError(m);
+    }
 }
