@@ -1,6 +1,6 @@
 /* commands.c
  * Code for directives
- * Copyright (C) 1991-2003,2004,2005,2006,2010,2011 Olly Betts
+ * Copyright (C) 1991-2003,2004,2005,2006,2010,2011,2012,2013 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -76,6 +76,7 @@ default_style(settings *s)
 {
    s->style = STYLE_NORMAL;
    s->ordering = default_order;
+   s->dash_for_anon_wall_station = fFalse;
 }
 
 static void
@@ -199,6 +200,23 @@ get_token(void)
 #endif
 }
 
+/* read word */
+static void
+get_word(void)
+{
+   s_zero(&buffer);
+   skipblanks();
+   while (!isBlank(ch) && !isEol(ch)) {
+      s_catchar(&buffer, &buf_len, (char)ch);
+      nextch();
+   }
+
+   if (!buffer) s_catchar(&buffer, &buf_len, '\0');
+#if 0
+   printf("get_word() got “%s”\n", buffer);
+#endif
+}
+
 /* match_tok() now uses binary chop
  * tab argument should be alphabetically sorted (ascending)
  */
@@ -223,7 +241,7 @@ match_tok(const sztok *tab, int tab_size)
 }
 
 typedef enum {
-   CMD_NULL = -1, CMD_BEGIN, CMD_CALIBRATE, CMD_CASE, CMD_COPYRIGHT,
+   CMD_NULL = -1, CMD_ALIAS, CMD_BEGIN, CMD_CALIBRATE, CMD_CASE, CMD_COPYRIGHT,
    CMD_DATA, CMD_DATE, CMD_DEFAULT, CMD_END, CMD_ENTRANCE, CMD_EQUATE,
    CMD_EXPORT, CMD_FIX, CMD_FLAGS, CMD_INCLUDE, CMD_INFER, CMD_INSTRUMENT,
    CMD_PREFIX, CMD_REQUIRE, CMD_SD, CMD_SET, CMD_SOLVE,
@@ -231,6 +249,7 @@ typedef enum {
 } cmds;
 
 static sztok cmd_tab[] = {
+     {"ALIAS",     CMD_ALIAS},
      {"BEGIN",     CMD_BEGIN},
      {"CALIBRATE", CMD_CALIBRATE},
      {"CASE",      CMD_CASE},
@@ -295,7 +314,8 @@ get_units(unsigned long qmask, bool percent_ok)
    get_token();
    units = match_tok(utab, TABSIZE(utab));
    if (units == UNITS_NULL) {
-      compile_error_skip(/*Unknown units “%s”*/35, buffer);
+      file.lpos += strlen(buffer);
+      compile_error_skip(-/*Unknown units “%s”*/35, buffer);
       return UNITS_NULL;
    }
    if (units == UNITS_PERCENT && percent_ok &&
@@ -304,7 +324,8 @@ get_units(unsigned long qmask, bool percent_ok)
    }
    if (((qmask & LEN_QMASK) && !TSTBIT(LEN_UMASK, units)) ||
        ((qmask & ANG_QMASK) && !TSTBIT(ANG_UMASK, units))) {
-      compile_error_skip(/*Invalid units “%s” for quantity*/37, buffer);
+      file.lpos += strlen(buffer);
+      compile_error_skip(-/*Invalid units “%s” for quantity*/37, buffer);
       return UNITS_NULL;
    }
    return units;
@@ -351,25 +372,30 @@ get_qlist(unsigned long mask_bad)
    int tok;
    filepos fp;
 
-   char default_buf[8] = "";
    while (1) {
       get_pos(&fp);
       get_token();
       tok = match_tok(qtab, TABSIZE(qtab));
+      if (tok == Q_DEFAULT && !(mask_bad & BIT(Q_DEFAULT))) {
+	  /* Only recognise DEFAULT if it is the first quantity, and then don't
+	   * look for any more. */
+	  if (qmask == 0)
+	      return BIT(Q_DEFAULT);
+	  break;
+      }
       /* bail out if we reach the table end with no match */
       if (tok == Q_NULL) break;
-      /* Preserve original case version of DeFAUlt token for error reports */
-      if (tok == Q_DEFAULT) strcpy(default_buf, buffer);
       qmask |= BIT(tok);
-      if (qmask != BIT(Q_DEFAULT) && (qmask & mask_bad)) {
-	 if (qmask & mask_bad & BIT(Q_DEFAULT)) strcpy(buffer, default_buf);
-	 compile_error_skip(/*Unknown instrument “%s”*/39, buffer);
+      if (qmask & mask_bad) {
+	 file.lpos += strlen(buffer);
+	 compile_error_skip(-/*Unknown instrument “%s”*/39, buffer);
 	 return 0;
       }
    }
 
    if (qmask == 0) {
-      compile_error_skip(/*Unknown quantity “%s”*/34, buffer);
+      file.lpos += strlen(buffer);
+      compile_error_skip(-/*Unknown quantity “%s”*/34, buffer);
    } else {
       set_pos(&fp);
    }
@@ -406,14 +432,16 @@ cmd_set(void)
    mask = match_tok(chartab, TABSIZE(chartab));
 
    if (mask == SPECIAL_UNKNOWN) {
-      compile_error_skip(/*Unknown character class “%s”*/42, buffer);
+      file.lpos += strlen(buffer);
+      compile_error_skip(-/*Unknown character class “%s”*/42, buffer);
       return;
    }
 
 #ifndef NO_DEPRECATED
    if (mask == SPECIAL_ROOT) {
       if (root_depr_count < 5) {
-	 compile_warning(/*ROOT is deprecated*/25);
+	 file.lpos += strlen(buffer);
+	 compile_warning(-/*ROOT is deprecated*/25);
 	 if (++root_depr_count == 5)
 	    compile_warning(/*Further uses of this deprecated feature will not be reported*/95);
       }
@@ -504,16 +532,38 @@ cmd_prefix(void)
     * deprecated and then that ROOT is...
     */
    if (prefix_depr_count < 5) {
-      compile_warning(/**prefix is deprecated - use *begin and *end instead*/6);
+      compile_warning(-/**prefix is deprecated - use *begin and *end instead*/6);
       if (++prefix_depr_count == 5)
 	 compile_warning(/*Further uses of this deprecated feature will not be reported*/95);
    }
-   tag = read_prefix_survey(fFalse, fTrue);
+   tag = read_prefix(PFX_SURVEY|PFX_ALLOW_ROOT);
    pcs->Prefix = tag;
    check_reentry(tag);
 }
 #endif
 
+static void
+cmd_alias(void)
+{
+   /* Currently only two forms are supported:
+    * *alias station - ..
+    * *alias station -
+    */
+   get_token();
+   if (strcmp(ucbuffer, "STATION") != 0)
+      goto bad;
+   get_word();
+   if (strcmp(buffer, "-") != 0)
+      goto bad;
+   get_word();
+   if (*buffer && strcmp(buffer, "..") != 0)
+      goto bad;
+   pcs->dash_for_anon_wall_station = (*buffer != '\0');
+   return;
+bad:
+   compile_error_skip(/*Bad *alias command*/397);
+}
+
 static void
 cmd_begin(void)
 {
@@ -526,7 +576,7 @@ cmd_begin(void)
    pcsNew->next = pcs;
    pcs = pcsNew;
 
-   tag = read_prefix_survey(fTrue, fTrue);
+   tag = read_prefix(PFX_SURVEY|PFX_OPT|PFX_ALLOW_ROOT|PFX_WARN_SEPARATOR);
    pcs->tag = tag;
    if (tag) {
       pcs->Prefix = tag;
@@ -578,19 +628,19 @@ cmd_end(void)
    pcs = pcsParent;
 
    /* note need to read using root *before* BEGIN */
-   tag = read_prefix_survey(fTrue, fTrue);
+   tag = read_prefix(PFX_SURVEY|PFX_OPT|PFX_ALLOW_ROOT);
    if (tag != tagBegin) {
       if (tag) {
 	 if (!tagBegin) {
 	    /* "*begin" / "*end foo" */
-	    compile_error_skip(/*Matching BEGIN tag has no prefix*/36);
+	    compile_error_skip(-/*Matching BEGIN tag has no prefix*/36);
 	 } else {
 	    /* tag mismatch */
-	    compile_error_skip(/*Prefix tag doesn’t match BEGIN*/193);
+	    compile_error_skip(-/*Prefix tag doesn’t match BEGIN*/193);
 	 }
       } else {
 	 /* close tag omitted; open tag given */
-	 compile_warning(/*Closing prefix omitted from END*/194);
+	 compile_warning(-/*Closing prefix omitted from END*/194);
       }
    }
 }
@@ -598,7 +648,7 @@ cmd_end(void)
 static void
 cmd_entrance(void)
 {
-   prefix *pfx = read_prefix_stn(fFalse, fFalse);
+   prefix *pfx = read_prefix(PFX_STATION);
    pfx->sflags |= BIT(SFLAGS_ENTRANCE);
 }
 
@@ -610,16 +660,16 @@ cmd_fix(void)
    static node *stnOmitAlready = NULL;
    real x, y, z;
    int nx, ny, nz;
-   bool fRef = 0;
    filepos fp;
 
-   fix_name = read_prefix_stn(fFalse, fTrue);
+   fix_name = read_prefix(PFX_STATION|PFX_ALLOW_ROOT);
    fix_name->sflags |= BIT(SFLAGS_FIXED);
 
    get_pos(&fp);
    get_token();
    if (strcmp(ucbuffer, "REFERENCE") == 0) {
-      fRef = 1;
+      /* suppress "unused fixed point" warnings for this station */
+      fix_name->sflags |= BIT(SFLAGS_USED);
    } else {
       if (*ucbuffer) set_pos(&fp);
    }
@@ -702,17 +752,12 @@ cmd_fix(void)
 		       , cxy, cyz, czx
 #endif
 		       );
-	    /* suppress "unused fixed point" warnings for this station */
-	    if (fRef && fix_name->shape == 1) fix_name->shape = -2;
 	 }
 	 return;
       }
       stn = StnFromPfx(fix_name);
    }
 
-   /* suppress "unused fixed point" warnings for this station */
-   if (fRef && fix_name->shape == 0) fix_name->shape = -1;
-
    if (!fixed(stn)) {
       POS(stn, 0) = x;
       POS(stn, 1) = y;
@@ -751,7 +796,8 @@ cmd_flags(void)
       flag = match_tok(flagtab, TABSIZE(flagtab));
       /* treat the second NOT in "NOT NOT" as an unknown flag */
       if (flag == FLAGS_UNKNOWN || (fNot && flag == FLAGS_NOT)) {
-	 compile_error(/*FLAG “%s” unknown*/68, buffer);
+	 file.lpos += strlen(buffer);
+	 compile_error(-/*FLAG “%s” unknown*/68, buffer);
 	 /* Recover from “*FLAGS NOT BOGUS SURFACE” by ignoring "NOT BOGUS" */
 	 fNot = fFalse;
       } else if (flag == FLAGS_NOT) {
@@ -765,9 +811,11 @@ cmd_flags(void)
    }
 
    if (fNot) {
-      compile_error(/*Expecting “DUPLICATE”, “SPLAY”, or “SURFACE”*/188);
+      file.lpos += strlen(buffer);
+      compile_error(-/*Expecting “DUPLICATE”, “SPLAY”, or “SURFACE”*/188);
    } else if (fEmpty) {
-      compile_error(/*Expecting “NOT”, “DUPLICATE”, “SPLAY”, or “SURFACE”*/189);
+      file.lpos += strlen(buffer);
+      compile_error(-/*Expecting “NOT”, “DUPLICATE”, “SPLAY”, or “SURFACE”*/189);
    }
 }
 
@@ -777,13 +825,13 @@ cmd_equate(void)
    prefix *name1, *name2;
    bool fOnlyOneStn = fTrue; /* to trap eg *equate entrance.6 */
 
-   name1 = read_prefix_stn_check_implicit(fFalse, fTrue);
+   name1 = read_prefix(PFX_STATION|PFX_ALLOW_ROOT|PFX_SUSPECT_TYPO);
    while (fTrue) {
       name2 = name1;
-      name1 = read_prefix_stn_check_implicit(fTrue, fTrue);
+      name1 = read_prefix(PFX_STATION|PFX_ALLOW_ROOT|PFX_SUSPECT_TYPO|PFX_OPT);
       if (name1 == NULL) {
 	 if (fOnlyOneStn) {
-	    compile_error_skip(/*Only one station in EQUATE command*/33);
+	    compile_error_skip(-/*Only one station in EQUATE command*/33);
 	 }
 	 return;
       }
@@ -825,7 +873,7 @@ cmd_export(void)
    prefix *pfx;
 
    fExportUsed = fTrue;
-   pfx = read_prefix_stn(fFalse, fFalse);
+   pfx = read_prefix(PFX_STATION);
    do {
       int depth = 0;
       {
@@ -859,7 +907,7 @@ cmd_export(void)
 	 }
 	 pfx->min_export = depth;
       }
-      pfx = read_prefix_stn(fTrue, fFalse);
+      pfx = read_prefix(PFX_STATION|PFX_OPT);
    } while (pfx);
 }
 
@@ -921,8 +969,8 @@ cmd_data(void)
 
    /* readings which may be given for each style */
    static const unsigned long mask[] = {
-      MASK_NORMAL, MASK_DIVING, MASK_CARTESIAN, MASK_CYLPOLAR, MASK_PASSAGE,
-      MASK_NOSURVEY
+      MASK_NORMAL, MASK_DIVING, MASK_CARTESIAN, MASK_CYLPOLAR, MASK_NOSURVEY,
+      MASK_PASSAGE
    };
 
    /* readings which may be omitted for each style */
@@ -931,8 +979,8 @@ cmd_data(void)
       BIT(Dir),
       0,
       BIT(Dir),
-      0, /* BIT(Left) | BIT(Right) | BIT(Up) | BIT(Down), */
-      0
+      0,
+      0 /* BIT(Left) | BIT(Right) | BIT(Up) | BIT(Down), */
    };
 
    /* all valid readings */
@@ -941,8 +989,8 @@ cmd_data(void)
       MASK_DIVING | BIT(Newline) | BIT(Ignore) | BIT(IgnoreAll) | BIT(End),
       MASK_CARTESIAN | BIT(Newline) | BIT(Ignore) | BIT(IgnoreAll) | BIT(End),
       MASK_CYLPOLAR | BIT(Newline) | BIT(Ignore) | BIT(IgnoreAll) | BIT(End),
-      MASK_PASSAGE | BIT(Ignore) | BIT(IgnoreAll) | BIT(End),
-      MASK_NOSURVEY | BIT(Ignore) | BIT(IgnoreAll) | BIT(End)
+      MASK_NOSURVEY | BIT(Ignore) | BIT(IgnoreAll) | BIT(End),
+      MASK_PASSAGE | BIT(Ignore) | BIT(IgnoreAll) | BIT(End)
    };
 #define STYLE_DEFAULT   -2
 #define STYLE_UNKNOWN   -1
@@ -982,7 +1030,8 @@ cmd_data(void)
    }
 
    if (style == STYLE_UNKNOWN) {
-      compile_error_skip(/*Data style “%s” unknown*/65, buffer);
+      file.lpos += strlen(buffer);
+      compile_error_skip(-/*Data style “%s” unknown*/65, buffer);
       return;
    }
 
@@ -993,7 +1042,8 @@ cmd_data(void)
    if (isOmit(ch)) {
       static int data_depr_count = 0;
       if (data_depr_count < 5) {
-	 compile_warning(/*“*data %s %c …” is deprecated - use “*data %s …” instead*/104,
+	 file.lpos += strlen(buffer);
+	 compile_warning(-/*“*data %s %c …” is deprecated - use “*data %s …” instead*/104,
 			 buffer, ch, buffer);
 	 if (++data_depr_count == 5)
 	    compile_warning(/*Further uses of this deprecated feature will not be reported*/95);
@@ -1015,7 +1065,8 @@ cmd_data(void)
       }
       /* Note: an unknown token is reported as trailing garbage */
       if (!TSTBIT(mask_all[style], d)) {
-	 compile_error_skip(/*Reading “%s” not allowed in data style “%s”*/63,
+	 file.lpos += strlen(buffer);
+	 compile_error_skip(-/*Reading “%s” not allowed in data style “%s”*/63,
 		       buffer, style_name);
 	 osfree(style_name);
 	 osfree(new_order);
@@ -1023,7 +1074,8 @@ cmd_data(void)
       }
       if (TSTBIT(mUsed, Newline) && TSTBIT(m_multi, d)) {
 	 /* e.g. "*data diving station newline tape depth compass" */
-	 compile_error_skip(/*Reading “%s” must occur before NEWLINE*/225, buffer);
+	 file.lpos += strlen(buffer);
+	 compile_error_skip(-/*Reading “%s” must occur before NEWLINE*/225, buffer);
 	 osfree(style_name);
 	 osfree(new_order);
 	 return;
@@ -1033,7 +1085,8 @@ cmd_data(void)
        */
       if (!((BIT(Ignore) | BIT(End) | BIT(IgnoreAll)) & BIT(d))) {
 	 if (TSTBIT(mUsed, d)) {
-	    compile_error_skip(/*Duplicate reading “%s”*/67, buffer);
+	    file.lpos += strlen(buffer);
+	    compile_error_skip(-/*Duplicate reading “%s”*/67, buffer);
 	    osfree(style_name);
 	    osfree(new_order);
 	    return;
@@ -1070,13 +1123,15 @@ cmd_data(void)
 	     case Newline:
 	       if (mUsed & ~m_multi) {
 		  /* e.g. "*data normal from to tape newline compass clino" */
-		  compile_error_skip(/*NEWLINE can only be preceded by STATION, DEPTH, and COUNT*/226);
+		  file.lpos += strlen(buffer);
+		  compile_error_skip(-/*NEWLINE can only be preceded by STATION, DEPTH, and COUNT*/226);
 		  osfree(style_name);
 		  osfree(new_order);
 		  return;
 	       }
 	       if (k == 0) {
-		  compile_error_skip(/*NEWLINE can’t be the first reading*/222);
+		  file.lpos += strlen(buffer);
+		  compile_error_skip(-/*NEWLINE can’t be the first reading*/222);
 		  osfree(style_name);
 		  osfree(new_order);
 		  return;
@@ -1087,7 +1142,8 @@ cmd_data(void)
 	    }
 	    if (fBad) {
 	       /* Not entirely happy with phrasing this... */
-	       compile_error_skip(/*Reading “%s” duplicates previous reading(s)*/77,
+	       file.lpos += strlen(buffer);
+	       compile_error_skip(-/*Reading “%s” duplicates previous reading(s)*/77,
 			     buffer);
 	       osfree(style_name);
 	       osfree(new_order);
@@ -1109,7 +1165,8 @@ cmd_data(void)
    } while (d != End);
 
    if (k >= 2 && new_order[k - 2] == Newline) {
-      compile_error_skip(/*NEWLINE can’t be the last reading*/223);
+      file.lpos += strlen(buffer);
+      compile_error_skip(-/*NEWLINE can’t be the last reading*/223);
       osfree(style_name);
       osfree(new_order);
       return;
@@ -1211,7 +1268,7 @@ cmd_units(void)
    unsigned long m; /* mask with bit x set to indicate quantity x specified */
    real factor;
 
-   qmask = get_qlist(BIT(Q_DEFAULT));
+   qmask = get_qlist(0);
    if (!qmask) return;
    if (qmask == BIT(Q_DEFAULT)) {
       default_units(pcs);
@@ -1220,7 +1277,7 @@ cmd_units(void)
 
    factor = read_numeric(fTrue, NULL);
    if (factor == 0.0) {
-      compile_error_skip(/**UNITS factor must be non-zero*/200);
+      compile_error_skip(-/**UNITS factor must be non-zero*/200);
       return;
    }
    if (factor == HUGE_REAL) factor = (real)1.0;
@@ -1245,7 +1302,7 @@ cmd_calibrate(void)
    unsigned long qmask, m;
    int quantity;
 
-   qmask = get_qlist(BIT(Q_DEFAULT)|BIT(Q_POS)|BIT(Q_PLUMB)|BIT(Q_LEVEL));
+   qmask = get_qlist(BIT(Q_POS)|BIT(Q_PLUMB)|BIT(Q_LEVEL));
    if (!qmask) return; /* error already reported */
 
    if (qmask == BIT(Q_DEFAULT)) {
@@ -1264,7 +1321,11 @@ cmd_calibrate(void)
    /* check for declination scale */
    /* perhaps "*calibrate declination XXX" should be "*declination XXX" ? */
    if (TSTBIT(qmask, Q_DECLINATION) && sc != 1.0) {
-      compile_error_skip(/*Scale factor must be 1.0 for DECLINATION*/40);
+      compile_error_skip(-/*Scale factor must be 1.0 for DECLINATION*/40);
+      return;
+   }
+   if (sc == 0.0) {
+      compile_error_skip(-/*Scale factor must be non-zero*/391);
       return;
    }
    for (quantity = 0, m = BIT(quantity); m <= qmask; quantity++, m <<= 1) {
@@ -1288,7 +1349,7 @@ cmd_default(void)
    static int default_depr_count = 0;
 
    if (default_depr_count < 5) {
-      compile_warning(/**DEFAULT is deprecated - use *CALIBRATE/DATA/SD/UNITS with argument DEFAULT instead*/20);
+      compile_warning(-/**DEFAULT is deprecated - use *CALIBRATE/DATA/SD/UNITS with argument DEFAULT instead*/20);
       if (++default_depr_count == 5)
 	 compile_warning(/*Further uses of this deprecated feature will not be reported*/95);
    }
@@ -1306,7 +1367,8 @@ cmd_default(void)
       default_units(pcs);
       break;
     default:
-      compile_error_skip(/*Unknown setting “%s”*/41, buffer);
+      file.lpos += strlen(buffer);
+      compile_error_skip(-/*Unknown setting “%s”*/41, buffer);
    }
 }
 #endif
@@ -1361,7 +1423,7 @@ cmd_sd(void)
    }
    sd = read_numeric(fFalse, NULL);
    if (sd <= (real)0.0) {
-      compile_error_skip(/*Standard deviation must be positive*/48);
+      compile_error_skip(-/*Standard deviation must be positive*/48);
       return;
    }
    units = get_units(qmask, fFalse);
@@ -1407,8 +1469,9 @@ cmd_case(void)
    if (setting != -1) {
       pcs->Case = setting;
    } else {
-      compile_error_skip(/*Found “%s”, expecting “PRESERVE”, “TOUPPER”, or “TOLOWER”*/10,
-		    buffer);
+      file.lpos += strlen(buffer);
+      compile_error_skip(-/*Found “%s”, expecting “PRESERVE”, “TOUPPER”, or “TOLOWER”*/10,
+			 buffer);
    }
 }
 
@@ -1436,13 +1499,15 @@ cmd_infer(void)
    get_token();
    setting = match_tok(infer_tab, TABSIZE(infer_tab));
    if (setting == INFER_NULL) {
-      compile_error_skip(/*Found “%s”, expecting “EQUATES”, “EXPORTS”, or “PLUMBS”*/31, buffer);
+      file.lpos += strlen(buffer);
+      compile_error_skip(-/*Found “%s”, expecting “EQUATES”, “EXPORTS”, or “PLUMBS”*/31, buffer);
       return;
    }
    get_token();
    on = match_tok(onoff_tab, TABSIZE(onoff_tab));
    if (on == -1) {
-      compile_error_skip(/*Found “%s”, expecting “ON” or “OFF”*/32, buffer);
+      file.lpos += strlen(buffer);
+      compile_error_skip(-/*Found “%s”, expecting “ON” or “OFF”*/32, buffer);
       return;
    }
 
@@ -1536,7 +1601,7 @@ cmd_date(void)
     days1 = days_since_1900(year, month ? month : 1, day ? day : 1);
 
     if (days1 > current_days_since_1900) {
-	compile_warning(/*Date is in the future!*/80);
+	compile_warning(-/*Date is in the future!*/80);
     }
 
     skipblanks();
@@ -1556,11 +1621,11 @@ cmd_date(void)
     days2 = days_since_1900(year, month, day);
 
     if (!implicit_range && days2 > current_days_since_1900) {
-	compile_warning(/*Date is in the future!*/80);
+	compile_warning(-/*Date is in the future!*/80);
     }
 
     if (days2 < days1) {
-	compile_error(/*End of date range is before the start*/81);
+	compile_error(-/*End of date range is before the start*/81);
     }
 
 read:
@@ -1572,6 +1637,7 @@ read:
 typedef void (*cmd_fn)(void);
 
 static cmd_fn cmd_funcs[] = {
+   cmd_alias,
    cmd_begin,
    cmd_calibrate,
    cmd_case,
@@ -1611,7 +1677,8 @@ handle_command(void)
    cmdtok = match_tok(cmd_tab, TABSIZE(cmd_tab));
 
    if (cmdtok < 0 || cmdtok >= (int)(sizeof(cmd_funcs) / sizeof(cmd_fn))) {
-      compile_error_skip(/*Unknown command “%s”*/12, buffer);
+      file.lpos += strlen(buffer);
+      compile_error_skip(-/*Unknown command “%s”*/12, buffer);
       return;
    }
 
