@@ -1,6 +1,6 @@
 /* datain.c
  * Reads in survey files, dealing with special characters, keywords & data
- * Copyright (C) 1991-2003,2005,2009,2010,2011,2012 Olly Betts
+ * Copyright (C) 1991-2003,2005,2009,2010,2011,2012,2013 Olly Betts
  * Copyright (C) 2004 Simeon Warner
  *
  * This program is free software; you can redistribute it and/or modify
@@ -95,50 +95,54 @@ push_back(int c)
       fatalerror_in_file(file.filename, 0, /*Error reading file*/18);
 }
 
+static void
+report_parent(parse * p) {
+    if (p->parent)
+	report_parent(p->parent);
+    /* Force re-report of include tree for further errors in
+     * parent files */
+    p->reported_where = fFalse;
+    fprintf(STDERR, msg(/*In file included from %s:%u:\n*/5), p->filename, p->line);
+}
+
 static void
 error_list_parent_files(void)
 {
    if (!file.reported_where && file.parent) {
-      parse *p = file.parent;
-      const char *m = msg(/*In file included from*/5);
-      size_t len = strlen(m);
-
-      fputs(m, STDERR);
-      m = msg(/*from*/3);
-
+      report_parent(file.parent);
       /* Suppress reporting of full include tree for further errors
        * in this file */
       file.reported_where = fTrue;
-
-      while (p) {
-	 /* Force re-report of include tree for further errors in
-	  * parent files */
-	 p->reported_where = fFalse;
-	 fprintf(STDERR, " %s:%d", p->filename, p->line);
-	 p = p->parent;
-	 if (p) fprintf(STDERR, ",\n%*s", (int)len, m);
-      }
-      fprintf(STDERR, ":\n");
    }
 }
 
 void
 compile_error(int en, ...)
 {
+   int col = 0;
    va_list ap;
    va_start(ap, en);
    error_list_parent_files();
-   v_report(1, file.filename, file.line, en, ap);
+   if (en < 0) {
+      en = -en;
+      col = ftell(file.fh) - file.lpos;
+   }
+   v_report(1, file.filename, file.line, col, en, ap);
    va_end(ap);
 }
 
 void
 compile_error_skip(int en, ...)
 {
+   int col = 0;
    va_list ap;
    va_start(ap, en);
    error_list_parent_files();
-   v_report(1, file.filename, file.line, en, ap);
+   if (en < 0) {
+      en = -en;
+      col = ftell(file.fh) - file.lpos;
+   }
+   v_report(1, file.filename, file.line, col, en, ap);
    va_end(ap);
    skipline();
 }
@@ -147,24 +151,28 @@ void
 compile_error_token(int en)
 {
    char *p = NULL;
-   static int len;
-   s_zero(&p);
+   int len = 0;
    skipblanks();
    while (!isBlank(ch) && !isEol(ch)) {
       s_catchar(&p, &len, (char)ch);
       nextch();
    }
-   compile_error_skip(en, p ? p : "");
+   compile_error(en, p ? p : "");
    osfree(p);
 }
 
 void
 compile_warning(int en, ...)
 {
+   int col = 0;
    va_list ap;
    va_start(ap, en);
    error_list_parent_files();
-   v_report(0, file.filename, file.line, en, ap);
+   if (en < 0) {
+      en = -en;
+      col = ftell(file.fh) - file.lpos;
+   }
+   v_report(0, file.filename, file.line, col, en, ap);
    va_end(ap);
 }
 
@@ -233,7 +241,7 @@ process_eol(void)
    skipblanks();
 
    if (!isEol(ch)) {
-      if (!isComm(ch)) compile_error(/*End of line not blank*/15);
+      if (!isComm(ch)) compile_error(-/*End of line not blank*/15);
       skipline();
    }
 
@@ -249,6 +257,7 @@ process_eol(void)
 	 break;
       }
    }
+   file.lpos = ftell(file.fh);
 }
 
 static bool
@@ -273,7 +282,6 @@ read_reading(reading r, bool f_optional)
 {
    int n_readings;
    q_quantity q;
-   VAL(r) = read_numeric(f_optional, &n_readings);
    switch (r) {
       case Tape: q = Q_LENGTH; break;
       case Comp: q = Q_BEARING; break;
@@ -293,6 +301,7 @@ read_reading(reading r, bool f_optional)
 	q = Q_NULL; /* Suppress compiler warning */;
 	BUG("Unexpected case");
    }
+   VAL(r) = read_numeric(f_optional, &n_readings);
    VAR(r) = var(q);
    if (n_readings > 1) VAR(r) /= sqrt(n_readings);
 }
@@ -365,6 +374,7 @@ data_file(const char *pth, const char *fnm)
       file.fh = fh;
       file.filename = filename;
       file.line = 1;
+      file.lpos = 0;
       file.reported_where = fFalse;
    }
 
@@ -458,6 +468,7 @@ data_file(const char *pth, const char *fnm)
 #ifdef HAVE_SETJMP_H
    /* errors in nested functions can longjmp here */
    if (setjmp(file.jbSkipLine)) {
+      skipline();
       process_eol();
    }
 #endif
@@ -597,7 +608,7 @@ data_file(const char *pth, const char *fnm)
 	    while (ch != ';' && ch != EOF) {
 	       prefix *name;
 	       nextch_handling_eol();
-	       name = read_prefix_stn(fTrue, fFalse);
+	       name = read_prefix(PFX_STATION|PFX_OPT);
 	       if (name) {
 		  skipblanks();
 		  if (ch == '[') {
@@ -889,17 +900,37 @@ process_normal(prefix *fr, prefix *to, bool fToFirst,
    fNoComp = handle_comp_units();
 
    if (ctype == CTYPE_READING) {
+      bool range_0_180;
+      real z;
       real diff_from_abs90;
       clin *= pcs->units[Q_GRADIENT];
       /* percentage scale */
       if (pcs->f_clino_percent) clin = atan(clin);
+      /* We want to warn if there's a reading which it would be impossible
+       * to have read from the instrument (e.g. on a -90 to 90 degree scale
+       * you can't read "96" (it's probably a typo for "69").  However, the
+       * gradient reading from a topofil is typically in the range 0 to 180,
+       * with 90 being horizontal.
+       *
+       * Really we should allow the valid range to be specified, but for now
+       * we infer it from the zero error - if this is within 45 degrees of
+       * 90 then we assume the range is 0 to 180.
+       */
+      z = pcs->z[Q_GRADIENT];
+      range_0_180 = (z > M_PI_4 && z < 3*M_PI_4);
       diff_from_abs90 = fabs(clin) - M_PI_2;
       if (diff_from_abs90 > EPSILON) {
-	 compile_warning(/*Clino reading over 90 degrees (absolute value)*/51);
+	 if (!range_0_180)
+	    compile_warning(/*Clino reading over 90 degrees (absolute value)*/51);
       } else if (TSTBIT(pcs->infer, INFER_PLUMBS) &&
 		 diff_from_abs90 >= -EPSILON) {
 	 ctype = CTYPE_INFERPLUMB;
       }
+      if (range_0_180 && ctype != CTYPE_INFERPLUMB) {
+	 /* FIXME: Warning message not ideal... */
+	 if (clin < 0.0 || clin - M_PI > EPSILON)
+	    compile_warning(/*Clino reading over 90 degrees (absolute value)*/51);
+      }
    }
 
    if (backctype == CTYPE_READING) {
@@ -1213,16 +1244,16 @@ data_cartesian(void)
       skipblanks();
       switch (*ordering) {
        case Fr:
-	 fr = read_prefix_stn(fFalse, fTrue);
+	 fr = read_prefix(PFX_STATION|PFX_ALLOW_ROOT);
 	 if (first_stn == End) first_stn = Fr;
 	 break;
        case To:
-	 to = read_prefix_stn(fFalse, fTrue);
+	 to = read_prefix(PFX_STATION|PFX_ALLOW_ROOT);
 	 if (first_stn == End) first_stn = To;
 	 break;
        case Station:
 	 fr = to;
-	 to = read_prefix_stn(fFalse, fFalse);
+	 to = read_prefix(PFX_STATION);
 	 first_stn = To;
 	 break;
        case Dx: case Dy: case Dz:
@@ -1363,20 +1394,24 @@ data_normal(void)
 
    again:
 
+   /* We clear these flags in the normal course of events, but if there's an
+    * error in a reading, we might not, so make sure it has been cleared here.
+    */
+   pcs->flags &= ~(BIT(FLAGS_ANON_ONE_END) | BIT(FLAGS_IMPLICIT_SPLAY));
    for (ordering = pcs->ordering; ; ordering++) {
       skipblanks();
       switch (*ordering) {
        case Fr:
-	  fr = read_prefix_stn(fFalse, fTrue);
+	  fr = read_prefix(PFX_STATION|PFX_ALLOW_ROOT|PFX_ANON);
 	  if (first_stn == End) first_stn = Fr;
 	  break;
        case To:
-	  to = read_prefix_stn(fFalse, fTrue);
+	  to = read_prefix(PFX_STATION|PFX_ALLOW_ROOT|PFX_ANON);
 	  if (first_stn == End) first_stn = To;
 	  break;
        case Station:
 	  fr = to;
-	  to = read_prefix_stn(fFalse, fFalse);
+	  to = read_prefix(PFX_STATION);
 	  first_stn = To;
 	  break;
        case Dir: {
@@ -1397,7 +1432,8 @@ data_normal(void)
 	     fRev = fTrue;
 	     break;
 	   default:
-	     compile_error_skip(/*Found “%s”, expecting “F” or “B”*/131,
+	     file.lpos += strlen(buffer);
+	     compile_error_skip(-/*Found “%s”, expecting “F” or “B”*/131,
 				buffer);
 	     process_eol();
 	     return;
@@ -1407,7 +1443,7 @@ data_normal(void)
        case Tape:
 	  read_reading(Tape, fFalse);
 	  if (VAL(Tape) < (real)0.0)
-	     compile_warning(/*Negative tape reading*/60);
+	     compile_warning(-/*Negative tape reading*/60);
 	  break;
        case Count:
 	  VAL(FrCount) = VAL(ToCount);
@@ -1424,28 +1460,21 @@ data_normal(void)
        case Comp: case BackComp:
 	  read_bearing_or_omit(*ordering);
 	  break;
-       case Clino:
-	  read_reading(Clino, fTrue);
-	  if (VAL(Clino) == HUGE_REAL) {
-	     VAL(Clino) = handle_plumb(&ctype);
-	     if (VAL(Clino) != HUGE_REAL) break;
-	     compile_error_token(/*Expecting numeric field, found “%s”*/9);
+       case Clino: case BackClino: {
+	  reading r = *ordering;
+	  clino_type * p_ctype = (r == Clino ? &ctype : &backctype);
+	  read_reading(r, fTrue);
+	  if (VAL(r) == HUGE_REAL) {
+	     VAL(r) = handle_plumb(p_ctype);
+	     if (VAL(r) != HUGE_REAL) break;
+	     compile_error_token(-/*Expecting numeric field, found “%s”*/9);
+	     skipline();
 	     process_eol();
 	     return;
 	  }
-	  ctype = CTYPE_READING;
-	  break;
-       case BackClino:
-	  read_reading(BackClino, fTrue);
-	  if (VAL(BackClino) == HUGE_REAL) {
-	     VAL(BackClino) = handle_plumb(&backctype);
-	     if (VAL(BackClino) != HUGE_REAL) break;
-	     compile_error_token(/*Expecting numeric field, found “%s”*/9);
-	     process_eol();
-	     return;
-	  }
-	  backctype = CTYPE_READING;
+	  *p_ctype = CTYPE_READING;
 	  break;
+       }
        case FrDepth: case ToDepth:
 	  read_reading(*ordering, fFalse);
 	  break;
@@ -1466,24 +1495,25 @@ data_normal(void)
 	  read_bearing_or_omit(BackComp);
 	  if (is_compass_NaN(VAL(BackComp))) VAL(BackComp) = HUGE_REAL;
 	  break;
-       case CompassDATClino:
-	  read_reading(Clino, fFalse);
-	  if (is_compass_NaN(VAL(Clino))) {
-	     VAL(Clino) = HUGE_REAL;
-	     ctype = CTYPE_OMIT;
+       case CompassDATClino: case CompassDATBackClino: {
+	  reading r;
+	  clino_type * p_ctype;
+	  if (*ordering == CompassDATClino) {
+	     r = Clino;
+	     p_ctype = &ctype;
 	  } else {
-	     ctype = CTYPE_READING;
+	     r = BackClino;
+	     p_ctype = &backctype;
 	  }
-	  break;
-       case CompassDATBackClino:
-	  read_reading(BackClino, fFalse);
-	  if (is_compass_NaN(VAL(BackClino))) {
-	     VAL(BackClino) = HUGE_REAL;
-	     backctype = CTYPE_OMIT;
+	  read_reading(r, fFalse);
+	  if (is_compass_NaN(VAL(r))) {
+	     VAL(r) = HUGE_REAL;
+	     *p_ctype = CTYPE_OMIT;
 	  } else {
-	     backctype = CTYPE_READING;
+	     *p_ctype = CTYPE_READING;
 	  }
 	  break;
+       }
        case CompassDATLeft: case CompassDATRight:
        case CompassDATUp: case CompassDATDown: {
 	  /* FIXME: need to actually make use of these entries! */
@@ -1531,6 +1561,8 @@ data_normal(void)
        case Newline:
 	  if (fr != NULL) {
 	     int r;
+	     int save_flags;
+	     int implicit_splay;
 	     if (fTopofil)
 		VAL(Tape) = VAL(ToCount) - VAL(FrCount);
 	     /* Note: frdepth == todepth test works regardless of fDepthChange
@@ -1553,6 +1585,12 @@ data_normal(void)
 		VAL(Tape) -= pcs->z[Q_LENGTH];
 		VAL(Tape) *= pcs->sc[Q_LENGTH];
 	     }
+	     implicit_splay = TSTBIT(pcs->flags, FLAGS_IMPLICIT_SPLAY);
+	     pcs->flags &= ~(BIT(FLAGS_ANON_ONE_END) | BIT(FLAGS_IMPLICIT_SPLAY));
+	     save_flags = pcs->flags;
+	     if (implicit_splay) {
+		pcs->flags |= BIT(FLAGS_SPLAY);
+	     }
 	     switch (pcs->style) {
 	      case STYLE_NORMAL:
 		r = process_normal(fr, to, (first_stn == To) ^ fRev,
@@ -1570,6 +1608,7 @@ data_normal(void)
 		r = 0; /* avoid warning */
 		BUG("bad style");
 	     }
+	     pcs->flags = save_flags;
 	     if (!r) skipline();
 
 	     /* Swap fr and to back to how they were for next line */
@@ -1606,6 +1645,8 @@ data_normal(void)
 	  /* fall through */
        case End:
 	  if (!fMulti) {
+	     int save_flags;
+	     int implicit_splay;
 	     /* Compass ignore flag is 'X' */
 	     if ((compass_dat_flags & BIT('X' - 'A'))) {
 		process_eol();
@@ -1633,6 +1674,12 @@ data_normal(void)
 		VAL(Tape) -= pcs->z[Q_LENGTH];
 		VAL(Tape) *= pcs->sc[Q_LENGTH];
 	     }
+	     implicit_splay = TSTBIT(pcs->flags, FLAGS_IMPLICIT_SPLAY);
+	     pcs->flags &= ~(BIT(FLAGS_ANON_ONE_END) | BIT(FLAGS_IMPLICIT_SPLAY));
+	     save_flags = pcs->flags;
+	     if (implicit_splay) {
+		pcs->flags |= BIT(FLAGS_SPLAY);
+	     }
 	     switch (pcs->style) {
 	      case STYLE_NORMAL:
 		process_normal(fr, to, (first_stn == To) ^ fRev,
@@ -1649,6 +1696,7 @@ data_normal(void)
 	      default:
 		BUG("bad style");
 	     }
+	     pcs->flags = save_flags;
 
 	     process_eol();
 	     return;
@@ -1693,15 +1741,16 @@ data_passage(void)
       skipblanks();
       switch (*ordering) {
        case Station:
-	 stn = read_prefix_stn(fFalse, fFalse);
+	 stn = read_prefix(PFX_STATION);
 	 break;
        case Left: case Right: case Up: case Down:
 	 read_reading(*ordering, fTrue);
          if (VAL(*ordering) == HUGE_REAL) {
             if (!isOmit(ch)) {
-	       compile_error_token(/*Expecting numeric field, found “%s”*/9);
+	       compile_error_token(-/*Expecting numeric field, found “%s”*/9);
+	    } else {
+	       nextch();
             }
-	    nextch();
             VAL(*ordering) = -1;
          }
 	 break;
@@ -1724,15 +1773,10 @@ static int
 process_nosurvey(prefix *fr, prefix *to, bool fToFirst)
 {
    nosurveylink *link;
-   int shape;
 
-   /* Suppress "unused fixed point" warnings for these stations
-    * We do this if it's a 0 or 1 node - 1 node might be an sdfix
-    */
-   shape = fr->shape;
-   if (shape == 0 || shape == 1) fr->shape = -1 - shape;
-   shape = to->shape;
-   if (shape == 0 || shape == 1) to->shape = -1 - shape;
+   /* Suppress "unused fixed point" warnings for these stations */
+   fr->sflags |= BIT(SFLAGS_USED);
+   to->sflags |= BIT(SFLAGS_USED);
 
    /* add to linked list which is dealt with after network is solved */
    link = osnew(nosurveylink);
@@ -1743,7 +1787,7 @@ process_nosurvey(prefix *fr, prefix *to, bool fToFirst)
       link->fr = StnFromPfx(fr);
       link->to = StnFromPfx(to);
    }
-   link->flags = pcs->flags;
+   link->flags = pcs->flags | (STYLE_NOSURVEY << FLAGS_STYLE_BIT0);
    link->meta = pcs->meta;
    if (pcs->meta) ++pcs->meta->ref_count;
    link->next = nosurveyhead;
@@ -1768,16 +1812,16 @@ data_nosurvey(void)
       skipblanks();
       switch (*ordering) {
        case Fr:
-	  fr = read_prefix_stn(fFalse, fTrue);
+	  fr = read_prefix(PFX_STATION|PFX_ALLOW_ROOT);
 	  if (first_stn == End) first_stn = Fr;
 	  break;
        case To:
-	  to = read_prefix_stn(fFalse, fTrue);
+	  to = read_prefix(PFX_STATION|PFX_ALLOW_ROOT);
 	  if (first_stn == End) first_stn = To;
 	  break;
        case Station:
 	  fr = to;
-	  to = read_prefix_stn(fFalse, fFalse);
+	  to = read_prefix(PFX_STATION);
 	  first_stn = To;
 	  break;
        case Ignore:
