@@ -1,8 +1,8 @@
 /* export.cc
- * Export to CAD-like formats (DXF, Sketch, SVG, EPS) and also Compass PLT.
+ * Export to CAD-like formats (DXF, Skencil, SVG, EPS) and also Compass PLT.
  */
 
-/* Copyright (C) 1994-2004,2005,2006,2008,2010,2011 Olly Betts
+/* Copyright (C) 1994-2004,2005,2006,2008,2010,2011,2012,2013 Olly Betts
  * Copyright (C) 2004 John Pybus (SVG Output code)
  *
  * This program is free software; you can redistribute it and/or modify
@@ -26,9 +26,12 @@
 #include <config.h>
 #endif
 
+#include "export.h"
+
 #include "wx.h"
 #include <wx/utils.h>
 #include "exportfilter.h"
+#include "gpx.h"
 #include "hpgl.h"
 #include "mainfrm.h"
 
@@ -49,40 +52,34 @@
 #include "debug.h"
 #include "filename.h"
 #include "hash.h"
-#include "img.h"
+#include "img_hosted.h"
 #include "message.h"
 #include "useful.h"
 
-/* default values - can be overridden with --htext and --msize respectively */
-#define TEXT_HEIGHT	0.6
-#define MARKER_SIZE	0.8
-
-#define GRID_SPACING	100
-
 #define POINTS_PER_INCH	72.0
 #define POINTS_PER_MM (POINTS_PER_INCH / MM_PER_INCH)
 
 #define SQRT_2		1.41421356237309504880168872420969
 
-#define LEGS 1
-#define SURF 2
-#define STNS 4
-#define LABELS 8
-
-static const char *layer_names[] = {
-   NULL,
-   "Legs",	// LEGS
-   "Surface",	// SURF
-   "Legs",	// LEGS|SURF
-   "Stations",	// STNS
-   NULL,
-   NULL,
-   NULL,
-   "Labels"	// LABELS
-};
-
-/* default to DXF */
-#define FMT_DEFAULT FMT_DXF
+static const char *layer_name(int mask) {
+    switch (mask) {
+	case LEGS: case LEGS|SURF:
+	    return "Legs";
+	case SURF:
+	    return "Surface";
+	case STNS:
+	    return "Stations";
+	case LABELS:
+	    return "Labels";
+	case XSECT:
+	    return "Cross-sections";
+	case WALL1: case WALL2: case WALLS:
+	    return "Walls";
+	case PASG:
+	    return "Passages";
+    }
+    return "";
+}
 
 /* bounds */
 static double min_x, min_y, min_z, max_x, max_y, max_z;
@@ -105,16 +102,33 @@ ExportFilter::passes() const
 }
 
 class DXF : public ExportFilter {
+    const char * to_close;
+    char pending[1024];
+
   public:
-    DXF() { }
+    DXF() : to_close(0) { pending[0] = '\0'; }
+    const int * passes() const;
     bool fopen(const char *fnm_out);
     void header(const char *);
     void line(const img_point *, const img_point *, bool, bool);
-    void label(const img_point *, const char *, bool);
+    void label(const img_point *, const char *, bool, int);
     void cross(const img_point *, bool);
+    void xsect(const img_point *, double, double, double);
+    void wall(const img_point *, double, double);
+    void passage(const img_point *, double, double, double);
+    void tube_end();
     void footer();
 };
 
+const int *
+DXF::passes() const
+{
+    static const int dxf_passes[] = {
+	PASG, XSECT, WALL1, WALL2, LEGS|SURF|STNS|LABELS, 0
+    };
+    return dxf_passes;
+}
+
 bool
 DXF::fopen(const char *fnm_out)
 {
@@ -257,7 +271,7 @@ DXF::line(const img_point *p1, const img_point *p, bool fSurface, bool fPendingM
 }
 
 void
-DXF::label(const img_point *p, const char *s, bool fSurface)
+DXF::label(const img_point *p, const char *s, bool fSurface, int)
 {
    /* write station label to dxf file */
    fprintf(fh, "0\nTEXT\n");
@@ -280,6 +294,73 @@ DXF::cross(const img_point *p, bool fSurface)
    fprintf(fh, "30\n%6.2f\n", p->z);
 }
 
+void
+DXF::xsect(const img_point *p, double angle, double d1, double d2)
+{
+   double s = sin(rad(angle));
+   double c = cos(rad(angle));
+   fprintf(fh, "0\nLINE\n");
+   fprintf(fh, "8\nCross-sections\n"); /* Layer */
+   fprintf(fh, "10\n%6.2f\n", p->x + c * d1);
+   fprintf(fh, "20\n%6.2f\n", p->y + s * d1);
+   fprintf(fh, "30\n%6.2f\n", p->z);
+   fprintf(fh, "11\n%6.2f\n", p->x - c * d2);
+   fprintf(fh, "21\n%6.2f\n", p->y - s * d2);
+   fprintf(fh, "31\n%6.2f\n", p->z);
+}
+
+void
+DXF::wall(const img_point *p, double angle, double d)
+{
+   if (!to_close) {
+       fprintf(fh, "0\nPOLYLINE\n");
+       fprintf(fh, "8\nWalls\n"); /* Layer */
+       fprintf(fh, "70\n0\n"); /* bit 0 == 0 => Open polyline */
+       to_close = "0\nSEQEND\n";
+   }
+   double s = sin(rad(angle));
+   double c = cos(rad(angle));
+   fprintf(fh, "0\nVERTEX\n");
+   fprintf(fh, "8\nWalls\n"); /* Layer */
+   fprintf(fh, "10\n%6.2f\n", p->x + c * d);
+   fprintf(fh, "20\n%6.2f\n", p->y + s * d);
+   fprintf(fh, "30\n%6.2f\n", p->z);
+}
+
+void
+DXF::passage(const img_point *p, double angle, double d1, double d2)
+{
+   fprintf(fh, "0\nSOLID\n");
+   fprintf(fh, "8\nPassages\n"); /* Layer */
+   double s = sin(rad(angle));
+   double c = cos(rad(angle));
+   double x1 = p->x + c * d1;
+   double y1 = p->y + s * d1;
+   double x2 = p->x - c * d2;
+   double y2 = p->y - s * d2;
+   if (*pending) {
+       fputs(pending, fh);
+       fprintf(fh, "12\n%6.2f\n22\n%6.2f\n32\n%6.2f\n"
+		   "13\n%6.2f\n23\n%6.2f\n33\n%6.2f\n",
+		   x1, y1, p->z,
+		   x2, y2, p->z);
+   }
+   sprintf(pending, "10\n%6.2f\n20\n%6.2f\n30\n%6.2f\n"
+		    "11\n%6.2f\n21\n%6.2f\n31\n%6.2f\n",
+		    x1, y1, p->z,
+		    x2, y2, p->z);
+}
+
+void
+DXF::tube_end()
+{
+   *pending = '\0';
+   if (to_close) {
+      fputs(to_close, fh);
+      to_close = NULL;
+   }
+}
+
 void
 DXF::footer()
 {
@@ -287,42 +368,42 @@ DXF::footer()
    fprintf(fh, "000\nEOF\n");
 }
 
-class Sketch : public ExportFilter {
+class Skencil : public ExportFilter {
   public:
-    Sketch() { }
+    Skencil() { }
     const int * passes() const;
     void header(const char *);
     void start_pass(int layer);
     void line(const img_point *, const img_point *, bool, bool);
-    void label(const img_point *, const char *, bool);
+    void label(const img_point *, const char *, bool, int);
     void cross(const img_point *, bool);
     void footer();
 };
 
 const int *
-Sketch::passes() const
+Skencil::passes() const
 {
-    static const int sketch_passes[] = { LEGS|SURF, STNS, LABELS, 0 };
-    return sketch_passes;
+    static const int skencil_passes[] = { LEGS|SURF, STNS, LABELS, 0 };
+    return skencil_passes;
 }
 
 void
-Sketch::header(const char *)
+Skencil::header(const char *)
 {
-   fprintf(fh, "##Sketch 1 2\n"); /* Sketch file version */
+   fprintf(fh, "##Sketch 1 2\n"); /* File format version */
    fprintf(fh, "document()\n");
    fprintf(fh, "layout((%.3f,%.3f),0)\n",
 	   (max_x - min_x) * factor, (max_y - min_y) * factor);
 }
 
 void
-Sketch::start_pass(int layer)
+Skencil::start_pass(int layer)
 {
-   fprintf(fh, "layer('%s',1,1,0,0,(0,0,0))\n", layer_names[layer]);
+   fprintf(fh, "layer('%s',1,1,0,0,(0,0,0))\n", layer_name(layer));
 }
 
 void
-Sketch::line(const img_point *p1, const img_point *p, bool fSurface, bool fPendingMove)
+Skencil::line(const img_point *p1, const img_point *p, bool fSurface, bool fPendingMove)
 {
    fSurface = fSurface; /* unused */
    if (fPendingMove) {
@@ -333,7 +414,7 @@ Sketch::line(const img_point *p1, const img_point *p, bool fSurface, bool fPendi
 }
 
 void
-Sketch::label(const img_point *p, const char *s, bool fSurface)
+Skencil::label(const img_point *p, const char *s, bool fSurface, int)
 {
    fSurface = fSurface; /* unused */
    fprintf(fh, "fp((0,0,0))\n");
@@ -350,23 +431,23 @@ Sketch::label(const img_point *p, const char *s, bool fSurface)
 }
 
 void
-Sketch::cross(const img_point *p, bool fSurface)
+Skencil::cross(const img_point *p, bool fSurface)
 {
    fSurface = fSurface; /* unused */
    fprintf(fh, "b()\n");
    fprintf(fh, "bs(%.3f,%.3f,%.3f)\n",
-	   p->x * factor - MARKER_SIZE, p->y * factor - MARKER_SIZE, 0.0);
+	   p->x * factor - marker_size, p->y * factor - marker_size, 0.0);
    fprintf(fh, "bs(%.3f,%.3f,%.3f)\n",
-	   p->x * factor + MARKER_SIZE, p->y * factor + MARKER_SIZE, 0.0);
+	   p->x * factor + marker_size, p->y * factor + marker_size, 0.0);
    fprintf(fh, "bn()\n");
    fprintf(fh, "bs(%.3f,%.3f,%.3f)\n",
-	   p->x * factor + MARKER_SIZE, p->y * factor - MARKER_SIZE, 0.0);
+	   p->x * factor + marker_size, p->y * factor - marker_size, 0.0);
    fprintf(fh, "bs(%.3f,%.3f,%.3f)\n",
-	   p->x * factor - MARKER_SIZE, p->y * factor + MARKER_SIZE, 0.0);
+	   p->x * factor - marker_size, p->y * factor + marker_size, 0.0);
 }
 
 void
-Sketch::footer(void)
+Skencil::footer(void)
 {
    fprintf(fh, "guidelayer('Guide Lines',1,0,0,1,(0,0,1))\n");
    if (grid) {
@@ -439,24 +520,31 @@ find_name(const img_point *p)
 }
 
 class SVG : public ExportFilter {
-    bool to_close;
+    const char * to_close;
     bool close_g;
+    char pending[1024];
 
   public:
-    SVG() : to_close(false), close_g(false) { }
+    SVG() : to_close(NULL), close_g(false) { pending[0] = '\0'; }
     const int * passes() const;
     void header(const char *);
     void start_pass(int layer);
     void line(const img_point *, const img_point *, bool, bool);
-    void label(const img_point *, const char *, bool);
+    void label(const img_point *, const char *, bool, int);
     void cross(const img_point *, bool);
+    void xsect(const img_point *, double, double, double);
+    void wall(const img_point *, double, double);
+    void passage(const img_point *, double, double, double);
+    void tube_end();
     void footer();
 };
 
 const int *
 SVG::passes() const
 {
-    static const int svg_passes[] = { LEGS|SURF, LABELS, STNS, 0 };
+    static const int svg_passes[] = {
+	PASG, LEGS|SURF, XSECT, WALL1, WALL2, LABELS, STNS, 0
+    };
     return svg_passes;
 }
 
@@ -478,7 +566,7 @@ SVG::header(const char *)
 	   width, unit, height, unit, width, height);
    fprintf(fh, "<g transform=\"translate(%.3f %.3f)\">\n",
 	   SVG_MARGIN - min_x * factor, SVG_MARGIN + max_y * factor);
-   to_close = false;
+   to_close = NULL;
    close_g = false;
 }
 
@@ -486,19 +574,25 @@ void
 SVG::start_pass(int layer)
 {
    if (to_close) {
-      fprintf(fh, "\"/>\n");
-      to_close = false;
+      fputs(to_close, fh);
+      to_close = NULL;
    }
    if (close_g) {
       fprintf(fh, "</g>\n");
    }
-   fprintf(fh, "<g id=\"%s\"", layer_names[layer]);
+   fprintf(fh, "<g id=\"%s\"", layer_name(layer));
    if (layer & LEGS)
       fprintf(fh, " stroke=\"black\" fill=\"none\" stroke-width=\"0.4px\"");
    else if (layer & STNS)
       fprintf(fh, " stroke=\"black\" fill=\"none\" stroke-width=\"0.05px\"");
    else if (layer & LABELS)
       fprintf(fh, " font-size=\"%.3fem\"", text_height);
+   else if (layer & XSECT)
+      fprintf(fh, " stroke=\"grey\" fill=\"none\" stroke-width=\"0.1px\"");
+   else if (layer & WALLS)
+      fprintf(fh, " stroke=\"black\" fill=\"none\" stroke-width=\"0.1px\"");
+   else if (layer & PASG)
+      fprintf(fh, " stroke=\"none\" fill=\"peru\"");
    fprintf(fh, ">\n");
 
    close_g = true;
@@ -510,16 +604,16 @@ SVG::line(const img_point *p1, const img_point *p, bool fSurface, bool fPendingM
    fSurface = fSurface; /* unused */
    if (fPendingMove) {
        if (to_close) {
-	   fprintf(fh, "\"/>\n");
+	   fputs(to_close, fh);
        }
        fprintf(fh, "<path d=\"M%.3f %.3f", p1->x * factor, p1->y * -factor);
    }
    fprintf(fh, "L%.3f %.3f", p->x * factor, p->y * -factor);
-   to_close = true;
+   to_close = "\"/>\n";
 }
 
 void
-SVG::label(const img_point *p, const char *s, bool fSurface)
+SVG::label(const img_point *p, const char *s, bool fSurface, int)
 {
    fSurface = fSurface; /* unused */
    fprintf(fh, "<text transform=\"translate(%.3f %.3f)\">",
@@ -534,20 +628,70 @@ SVG::cross(const img_point *p, bool fSurface)
 {
    fSurface = fSurface; /* unused */
    fprintf(fh, "<circle id=\"%s\" cx=\"%.3f\" cy=\"%.3f\" r=\"%.3f\"/>\n",
-	   find_name(p), p->x * factor, p->y * -factor, MARKER_SIZE * SQRT_2);
+	   find_name(p), p->x * factor, p->y * -factor, marker_size * SQRT_2);
    fprintf(fh, "<path d=\"M%.3f %.3fL%.3f %.3fM%.3f %.3fL%.3f %.3f\"/>\n",
-	   p->x * factor - MARKER_SIZE, p->y * -factor - MARKER_SIZE,
-	   p->x * factor + MARKER_SIZE, p->y * -factor + MARKER_SIZE,
-	   p->x * factor + MARKER_SIZE, p->y * -factor - MARKER_SIZE,
-	   p->x * factor - MARKER_SIZE, p->y * -factor + MARKER_SIZE );
+	   p->x * factor - marker_size, p->y * -factor - marker_size,
+	   p->x * factor + marker_size, p->y * -factor + marker_size,
+	   p->x * factor + marker_size, p->y * -factor - marker_size,
+	   p->x * factor - marker_size, p->y * -factor + marker_size);
+}
+
+void
+SVG::xsect(const img_point *p, double angle, double d1, double d2)
+{
+   double s = sin(rad(angle));
+   double c = cos(rad(angle));
+   fprintf(fh, "<path d=\"M%.3f %.3fL%.3f %.3f\"/>\n",
+	   (p->x + c * d1) * factor, (p->y + s * d1) * -factor,
+	   (p->x - c * d2) * factor, (p->y - s * d2) * -factor);
+}
+
+void
+SVG::wall(const img_point *p, double angle, double d)
+{
+   if (!to_close) {
+       fprintf(fh, "<path d=\"M");
+       to_close = "\"/>\n";
+   } else {
+       fprintf(fh, "L");
+   }
+   double s = sin(rad(angle));
+   double c = cos(rad(angle));
+   fprintf(fh, "%.3f %.3f", (p->x + c * d) * factor, (p->y + s * d) * -factor);
+}
+
+void
+SVG::passage(const img_point *p, double angle, double d1, double d2)
+{
+   double s = sin(rad(angle));
+   double c = cos(rad(angle));
+   double x1 = (p->x + c * d1) * factor;
+   double y1 = (p->y + s * d1) * -factor;
+   double x2 = (p->x - c * d2) * factor;
+   double y2 = (p->y - s * d2) * -factor;
+   if (*pending) {
+       fputs(pending, fh);
+       fprintf(fh, "L%.3f %.3fL%.3f %.3fZ\"/>\n", x2, y2, x1, y1);
+   }
+   sprintf(pending, "<path d=\"M%.3f %.3fL%.3f %.3f", x1, y1, x2, y2);
+}
+
+void
+SVG::tube_end()
+{
+   *pending = '\0';
+   if (to_close) {
+      fputs(to_close, fh);
+      to_close = NULL;
+   }
 }
 
 void
 SVG::footer()
 {
    if (to_close) {
-      fprintf(fh, "\"/>\n");
-      to_close = false;
+      fputs(to_close, fh);
+      to_close = NULL;
    }
    if (close_g) {
       fprintf(fh, "</g>\n");
@@ -566,7 +710,7 @@ class PLT : public ExportFilter {
     const int * passes() const;
     void header(const char *);
     void line(const img_point *, const img_point *, bool, bool);
-    void label(const img_point *, const char *, bool);
+    void label(const img_point *, const char *, bool, int);
     void footer();
 };
 
@@ -638,7 +782,7 @@ PLT::find_name_plt(const img_point *p)
 }
 
 void
-PLT::label(const img_point *p, const char *s, bool fSurface)
+PLT::label(const img_point *p, const char *s, bool fSurface, int)
 {
    fSurface = fSurface; /* unused */
    set_name(p, s);
@@ -661,7 +805,7 @@ class EPS : public ExportFilter {
     EPS() { }
     void header(const char *);
     void line(const img_point *, const img_point *, bool, bool);
-    void label(const img_point *, const char *, bool);
+    void label(const img_point *, const char *, bool, int);
     void cross(const img_point *, bool);
     void footer();
 };
@@ -911,7 +1055,7 @@ EPS::line(const img_point *p1, const img_point *p, bool fSurface, bool fPendingM
 }
 
 void
-EPS::label(const img_point *p, const char *s, bool fSurface)
+EPS::label(const img_point *p, const char *s, bool /*fSurface*/, int)
 {
    fprintf(fh, "%.2f %.2f M\n", p->x, p->y);
    PUTC('(', fh);
@@ -946,87 +1090,60 @@ EPS::footer(void)
 	 "%%EOF\n", fh);
 }
 
-typedef enum {
-    FMT_DXF = 0, FMT_SVG, FMT_SKETCH, FMT_PLT, FMT_EPS, FMT_HPGL, FMT_ENDMARKER
-} export_format;
-static const char *extensions[] = { "dxf", "svg", "sk", "plt", "eps", "hpgl" };
-
 bool
 Export(const wxString &fnm_out, const wxString &title, const MainFrm * mainfrm,
-       double pan, double tilt, bool labels, bool crosses, bool legs,
-       bool surface)
+       double pan, double tilt, int show_mask, export_format format,
+       const char * input_projection,
+       double grid_, double text_height_, double marker_size_)
 {
    int fPendingMove = 0;
    img_point p, p1;
    double s = 0, c = 0;
-   export_format format = FMT_DXF;
    const int *pass;
    bool elevation = (tilt == 0.0);
 
-   /* Defaults */
-   grid = 0;
-   text_height = TEXT_HEIGHT;
-   marker_size = MARKER_SIZE;
-   // FIXME: allow these to be set from aven somehow!
-#if 0
-       case 'g': /* Grid */
-	 if (optarg) {
-	    grid = cmdline_double_arg();
-	 } else {
-	    grid = (double)GRID_SPACING;
-	 }
-	 break;
+   grid = grid_;
+   text_height = text_height_;
+   marker_size = marker_size_;
+
+#if 0 // FIXME: allow these to be set from aven somehow!
        case 'r': /* Reduction factor */
 	 scale = cmdline_double_arg();
 	 break;
-       case 't': /* Text height */
-	 text_height = cmdline_double_arg();
-	 break;
-       case 'm': /* Marker size */
-	 marker_size = cmdline_double_arg();
-	 break;
        case 's':
 	 survey = optarg;
 	 break;
 #endif
 
-   {
-      size_t i;
-      size_t len = fnm_out.length();
-      for (i = 0; i < FMT_ENDMARKER; ++i) {
-	 size_t l = strlen(extensions[i]);
-	 if (len > l + 1 && fnm_out[len - l - 1] == FNM_SEP_EXT &&
-	     strcasecmp((const char *)fnm_out.mb_str() + len - l, extensions[i]) == 0) {
-	    format = export_format(i);
-	    break;
-	 }
-      }
-   }
-
    ExportFilter * filt;
    switch (format) {
        case FMT_DXF:
 	   filt = new DXF;
 	   break;
-       case FMT_SKETCH:
-	   filt = new Sketch;
-	   factor = POINTS_PER_MM * 1000.0 / scale;
-	   break;
-       case FMT_PLT:
-	   filt = new PLT;
-	   break;
-       case FMT_SVG:
-	   filt = new SVG;
-	   factor = 1000.0 / scale;
-	   break;
        case FMT_EPS:
 	   filt = new EPS;
 	   factor = POINTS_PER_MM * 1000.0 / scale;
 	   break;
+       case FMT_GPX:
+	   filt = new GPX(input_projection);
+	   show_mask |= FULL_COORDS;
+	   break;
        case FMT_HPGL:
 	   filt = new HPGL;
 	   // factor = POINTS_PER_MM * 1000.0 / scale;
 	   break;
+       case FMT_PLT:
+	   filt = new PLT;
+	   show_mask |= FULL_COORDS;
+	   break;
+       case FMT_SK:
+	   filt = new Skencil;
+	   factor = POINTS_PER_MM * 1000.0 / scale;
+	   break;
+       case FMT_SVG:
+	   filt = new SVG;
+	   factor = 1000.0 / scale;
+	   break;
        default:
 	   return false;
    }
@@ -1110,21 +1227,41 @@ Export(const wxString &fnm_out, const wxString &title, const MainFrm * mainfrm,
       max_x = max_y = max_z = 0;
    }
 
+   double x_offset, y_offset, z_offset;
+   if (show_mask & FULL_COORDS) {
+       // Full coordinates.
+       x_offset = mainfrm->GetOffset().GetX();
+       y_offset = mainfrm->GetOffset().GetY();
+       z_offset = mainfrm->GetOffset().GetZ();
+   } else if (show_mask & CENTRED) {
+       // Centred.
+       x_offset = (min_x + max_x) * -0.5;
+       y_offset = (min_y + max_y) * -0.5;
+       z_offset = (min_z + max_z) * -0.5;
+   } else {
+       // Origin at lowest SW corner.
+       x_offset = -min_x;
+       y_offset = -min_y;
+       z_offset = -min_z;
+   }
+   min_x += x_offset;
+   max_x += x_offset;
+   min_y += y_offset;
+   max_y += y_offset;
+   min_z += z_offset;
+   max_z += z_offset;
+
    /* Header */
    filt->header(title.mb_str());
 
    p1.x = p1.y = p1.z = 0; /* avoid compiler warning */
 
    for (pass = filt->passes(); *pass; ++pass) {
-      bool legs_this_pass = ((*pass & LEGS) && legs);
-      bool surf_this_pass = ((*pass & SURF) && legs && surface);
-      bool crosses_this_pass = ((*pass & STNS) && crosses);
-      bool labels_this_pass = ((*pass & LABELS) && labels);
-      if (!(legs_this_pass || surf_this_pass || crosses_this_pass ||
-	    labels_this_pass))
+      int pass_mask = show_mask & *pass;
+      if (!pass_mask)
 	  continue;
       filt->start_pass(*pass);
-      if (legs_this_pass) {
+      if (pass_mask & LEGS) {
 	  trav = mainfrm->traverses_begin();
 	  tend = mainfrm->traverses_end();
 	  for ( ; trav != tend; ++trav) {
@@ -1132,20 +1269,9 @@ Export(const wxString &fnm_out, const wxString &title, const MainFrm * mainfrm,
 	     vector<PointInfo>::const_iterator pos = trav->begin();
 	     vector<PointInfo>::const_iterator end = trav->end();
 	     for ( ; pos != end; ++pos) {
-		 p.x = pos->GetX();
-		 p.y = pos->GetY();
-		 p.z = pos->GetZ();
-
-		 if (format == FMT_SKETCH) {
-		     p.x -= min_x;
-		     p.y -= min_y;
-		     p.z -= min_z;
-		 } else if (format == FMT_HPGL) {
-		     p.x -= (min_x + max_x) / 2;
-		     p.y -= (min_y + max_y) / 2;
-		     p.z -= (min_z + max_z) / 2;
-		 }
-
+		 p.x = pos->GetX() + x_offset;
+		 p.y = pos->GetY() + y_offset;
+		 p.z = pos->GetZ() + z_offset;
 		 if (elevation) {
 		     double xnew = p.x * c - p.y * s;
 		     double znew = - p.x * s - p.y * c;
@@ -1171,7 +1297,7 @@ Export(const wxString &fnm_out, const wxString &title, const MainFrm * mainfrm,
 	     }
 	 }
       }
-      if (surf_this_pass) {
+      if (pass_mask & SURF) {
 	  trav = mainfrm->surface_traverses_begin();
 	  tend = mainfrm->surface_traverses_end();
 	  for ( ; trav != tend; ++trav) {
@@ -1179,19 +1305,9 @@ Export(const wxString &fnm_out, const wxString &title, const MainFrm * mainfrm,
 	     vector<PointInfo>::const_iterator pos = trav->begin();
 	     vector<PointInfo>::const_iterator end = trav->end();
 	     for ( ; pos != end; ++pos) {
-		 p.x = pos->GetX();
-		 p.y = pos->GetY();
-		 p.z = pos->GetZ();
-
-		 if (format == FMT_SKETCH) {
-		     p.x -= min_x;
-		     p.y -= min_y;
-		     p.z -= min_z;
-		 } else if (format == FMT_HPGL) {
-		     p.x -= (min_x + max_x) / 2;
-		     p.y -= (min_y + max_y) / 2;
-		     p.z -= (min_z + max_z) / 2;
-		 }
+		 p.x = pos->GetX() + x_offset;
+		 p.y = pos->GetY() + y_offset;
+		 p.z = pos->GetZ() + z_offset;
 
 		 if (elevation) {
 		     double xnew = p.x * c - p.y * s;
@@ -1218,23 +1334,13 @@ Export(const wxString &fnm_out, const wxString &title, const MainFrm * mainfrm,
 	     }
 	 }
       }
-      if (crosses_this_pass || labels_this_pass) {
+      if (pass_mask & (STNS|LABELS|ENTS|FIXES|EXPORTS)) {
 	  list<LabelInfo*>::const_iterator pos = mainfrm->GetLabels();
 	  list<LabelInfo*>::const_iterator end = mainfrm->GetLabelsEnd();
 	  for ( ; pos != end; ++pos) {
-	      p.x = (*pos)->GetX();
-	      p.y = (*pos)->GetY();
-	      p.z = (*pos)->GetZ();
-
-	      if (format == FMT_SKETCH) {
-		  p.x -= min_x;
-		  p.y -= min_y;
-		  p.z -= min_z;
-	      } else if (format == FMT_HPGL) {
-		  p.x -= (min_x + max_x) / 2;
-		  p.y -= (min_y + max_y) / 2;
-		  p.z -= (min_z + max_z) / 2;
-	      }
+	      p.x = (*pos)->GetX() + x_offset;
+	      p.y = (*pos)->GetY() + y_offset;
+	      p.z = (*pos)->GetZ() + z_offset;
 
 	      if (elevation) {
 		  double xnew = p.x * c - p.y * s;
@@ -1246,15 +1352,67 @@ Export(const wxString &fnm_out, const wxString &title, const MainFrm * mainfrm,
 #ifdef DEBUG_CAD3D
 	      printf("label '%s' at %9.2f %9.2f %9.2f\n",(*pos)->GetText(),x,y,z);
 #endif
-	      /* Use !UNDERGROUND as the criterion - we want stations where
-	       * a surface and underground survey meet to be in the
-	       * underground layer */
-	      if (labels_this_pass)
-		  filt->label(&p, (*pos)->GetText().mb_str(), !(*pos)->IsUnderground());
-	      if (crosses_this_pass)
+	      int type = 0;
+	      if ((pass_mask & ENTS) && (*pos)->IsEntrance()) {
+		  type = ENTS;
+	      } else if ((pass_mask & FIXES) && (*pos)->IsFixedPt()) {
+		  type = FIXES;
+	      } else if ((pass_mask & EXPORTS) && (*pos)->IsExportedPt())  {
+		  type = EXPORTS;
+	      } else if (pass_mask & LABELS) {
+		  type = LABELS;
+	      }
+	      if (type) {
+		  const char * text = (*pos)->GetText().mb_str();
+		  /* Use !UNDERGROUND as the criterion - we want stations where
+		   * a surface and underground survey meet to be in the
+		   * underground layer */
+		  filt->label(&p, text, !(*pos)->IsUnderground(), type);
+	      }
+	      if (pass_mask & STNS)
 		  filt->cross(&p, !(*pos)->IsUnderground());
 	  }
       }
+      if (pass_mask & (XSECT|WALLS|PASG)) {
+	  list<vector<XSect> >::const_iterator tube = mainfrm->tubes_begin();
+	  list<vector<XSect> >::const_iterator tube_end = mainfrm->tubes_end();
+	  for ( ; tube != tube_end; ++tube) {
+	      vector<XSect>::const_iterator pos = tube->begin();
+	      vector<XSect>::const_iterator end = tube->end();
+	      for ( ; pos != end; ++pos) {
+		  const XSect & xs = *pos;
+		  p.x = xs.GetX() + x_offset;
+		  p.y = xs.GetY() + y_offset;
+		  p.z = xs.GetZ() + z_offset;
+
+		  if (elevation) {
+		      double xnew = p.x * c - p.y * s;
+		      double znew = - p.x * s - p.y * c;
+		      p.y = p.z;
+		      p.z = znew;
+		      p.x = xnew;
+		      if (pass_mask & XSECT)
+			  filt->xsect(&p, 0, xs.GetU(), xs.GetD());
+		      if (pass_mask & WALL1)
+			  filt->wall(&p, 0, xs.GetU());
+		      if (pass_mask & WALL2)
+			  filt->wall(&p, 180, xs.GetD());
+		      if (pass_mask & PASG)
+			  filt->passage(&p, 0, xs.GetU(), xs.GetD());
+		  } else {
+		      if (pass_mask & XSECT)
+			  filt->xsect(&p, xs.get_right_bearing() + 180, xs.GetL(), xs.GetR());
+		      if (pass_mask & WALL1)
+			  filt->wall(&p, xs.get_right_bearing() + 180, xs.GetL());
+		      if (pass_mask & WALL2)
+			  filt->wall(&p, xs.get_right_bearing(), xs.GetR());
+		      if (pass_mask & PASG)
+			  filt->passage(&p, xs.get_right_bearing() + 180, xs.GetL(), xs.GetR());
+		  }
+	      }
+	      filt->tube_end();
+	  }
+      }
    }
    filt->footer();
    delete filt;
