@@ -91,7 +91,7 @@ static const gla_colour NAME_COLOUR = col_GREEN;
 // vector for lighting angle
 static const Vector3 light(.577, .577, .577);
 
-BEGIN_EVENT_TABLE(GfxCore, wxGLCanvas)
+BEGIN_EVENT_TABLE(GfxCore, GLACanvas)
     EVT_PAINT(GfxCore::OnPaint)
     EVT_LEFT_DOWN(GfxCore::OnLButtonDown)
     EVT_LEFT_UP(GfxCore::OnLButtonUp)
@@ -122,6 +122,7 @@ GfxCore::GfxCore(MainFrm* parent, wxWindow* parent_win, GUIControl* control) :
     m_SwitchingTo(0),
     m_Crosses(false),
     m_Legs(true),
+    m_Splays(SPLAYS_SHOW_FADED),
     m_Names(false),
     m_Scalebar(true),
     m_ColourKey(true),
@@ -274,6 +275,11 @@ bool GfxCore::HasUndergroundLegs() const
     return m_Parent->HasUndergroundLegs();
 }
 
+bool GfxCore::HasSplays() const
+{
+    return m_Parent->HasSplays();
+}
+
 bool GfxCore::HasSurfaceLegs() const
 {
     return m_Parent->HasSurfaceLegs();
@@ -293,7 +299,7 @@ void GfxCore::UpdateBlobs()
 //  Event handlers
 //
 
-void GfxCore::OnLeaveWindow(wxMouseEvent& event) {
+void GfxCore::OnLeaveWindow(wxMouseEvent&) {
     SetHere();
     ClearCoords();
 }
@@ -1129,7 +1135,7 @@ void GfxCore::OnSize(wxSizeEvent& event)
 	return;
     }
 
-    GLACanvas::OnSize(event);
+    event.Skip();
 
     if (m_DoneFirstShow) {
 	TryToFreeArrays();
@@ -1208,7 +1214,13 @@ bool GfxCore::Animate()
     double t;
     if (movie) {
 	ReadPixels(movie->GetWidth(), movie->GetHeight(), movie->GetBuffer());
-	movie->AddFrame();
+	if (!movie->AddFrame()) {
+	    wxGetApp().ReportError(wxString(movie->get_error_string(), wxConvUTF8));
+	    delete movie;
+	    movie = NULL;
+	    presentation_mode = 0;
+	    return false;
+	}
 	t = 1.0 / 25.0; // 25 frames per second
     } else {
 	t = timer.Time() * 1.0e-3;
@@ -1233,6 +1245,9 @@ bool GfxCore::Animate()
 	    if (!next_mark.is_valid()) {
 		SetView(prev_mark);
 		presentation_mode = 0;
+		if (movie && !movie->Close()) {
+		    wxGetApp().ReportError(wxString(movie->get_error_string(), wxConvUTF8));
+		}
 		delete movie;
 		movie = NULL;
 		break;
@@ -1382,8 +1397,7 @@ void GfxCore::RefreshLine(const Point &a, const Point &b, const Point &c)
 	    d = min(d, y - MARGIN);
 	}
     }
-    const wxRect R(l, d, r - l, u - d);
-    Refresh(false, &R);
+    RefreshRect(wxRect(l, d, r - l, u - d), false);
 }
 
 void GfxCore::SetHere()
@@ -1693,13 +1707,11 @@ void GfxCore::RedrawIndicators()
 {
     // Redraw the compass and clino indicators.
 
-    const wxRect r(GetXSize() - INDICATOR_OFFSET_X - INDICATOR_BOX_SIZE*2 -
-		     INDICATOR_GAP,
-		   GetYSize() - INDICATOR_OFFSET_Y - INDICATOR_BOX_SIZE,
-		   INDICATOR_BOX_SIZE*2 + INDICATOR_GAP,
-		   INDICATOR_BOX_SIZE);
-
-    Refresh(false, &r);
+    RefreshRect(wxRect(GetXSize() - INDICATOR_OFFSET_X - INDICATOR_BOX_SIZE*2 -
+		       INDICATOR_GAP,
+		       GetYSize() - INDICATOR_OFFSET_Y - INDICATOR_BOX_SIZE,
+		       INDICATOR_BOX_SIZE*2 + INDICATOR_GAP,
+		       INDICATOR_BOX_SIZE), false);
 }
 
 void GfxCore::StartRotation()
@@ -2002,8 +2014,21 @@ void GfxCore::GenerateDisplayList()
     // Generate the display list for the underground legs.
     list<traverse>::const_iterator trav = m_Parent->traverses_begin();
     list<traverse>::const_iterator tend = m_Parent->traverses_end();
+
+    if (m_Splays == SPLAYS_SHOW_FADED) {
+	SetAlpha(0.4);
+	while (trav != tend) {
+	    if ((*trav).isSplay)
+		(this->*AddPoly)(*trav);
+	    ++trav;
+	}
+	SetAlpha(1.0);
+	trav = m_Parent->traverses_begin();
+    }
+
     while (trav != tend) {
-	(this->*AddPoly)(*trav);
+	if (m_Splays == SPLAYS_SHOW_NORMAL || !(*trav).isSplay)
+	    (this->*AddPoly)(*trav);
 	++trav;
     }
 }
@@ -2011,8 +2036,8 @@ void GfxCore::GenerateDisplayList()
 void GfxCore::GenerateDisplayListTubes()
 {
     // Generate the display list for the tubes.
-    list<vector<XSect> >::const_iterator trav = m_Parent->tubes_begin();
-    list<vector<XSect> >::const_iterator tend = m_Parent->tubes_end();
+    list<vector<XSect> >::iterator trav = m_Parent->tubes_begin();
+    list<vector<XSect> >::iterator tend = m_Parent->tubes_end();
     while (trav != tend) {
 	SkinPassage(*trav);
 	++trav;
@@ -2505,7 +2530,7 @@ void GfxCore::AddPolylineError(const traverse & centreline)
 }
 
 void
-GfxCore::SkinPassage(const vector<XSect> & centreline)
+GfxCore::SkinPassage(vector<XSect> & centreline, bool draw)
 {
     assert(centreline.size() > 1);
     Vector3 U[4];
@@ -2514,11 +2539,11 @@ GfxCore::SkinPassage(const vector<XSect> & centreline)
 
 //  FIXME: it's not simple to set the colour of a tube based on error...
 //    static_E_hack = something...
-    vector<XSect>::const_iterator i = centreline.begin();
+    vector<XSect>::iterator i = centreline.begin();
     vector<XSect>::size_type segment = 0;
     while (i != centreline.end()) {
 	// get the coordinates of this vertex
-	const XSect & pt_v = *i++;
+	XSect & pt_v = *i++;
 
 	double z_pitch_adjust = 0.0;
 	bool cover_end = false;
@@ -2695,15 +2720,17 @@ GfxCore::SkinPassage(const vector<XSect> & centreline)
 	v[2] = pt_v + right * r - up * d;
 	v[3] = pt_v - right * l - up * d;
 
-	if (segment > 0) {
-	    (this->*AddQuad)(v[0], v[1], U[1], U[0]);
-	    (this->*AddQuad)(v[2], v[3], U[3], U[2]);
-	    (this->*AddQuad)(v[1], v[2], U[2], U[1]);
-	    (this->*AddQuad)(v[3], v[0], U[0], U[3]);
-	}
+	if (draw) {
+	    if (segment > 0) {
+		(this->*AddQuad)(v[0], v[1], U[1], U[0]);
+		(this->*AddQuad)(v[2], v[3], U[3], U[2]);
+		(this->*AddQuad)(v[1], v[2], U[2], U[1]);
+		(this->*AddQuad)(v[3], v[0], U[0], U[3]);
+	    }
 
-	if (cover_end) {
-	    (this->*AddQuad)(v[3], v[2], v[1], v[0]);
+	    if (cover_end) {
+		(this->*AddQuad)(v[3], v[2], v[1], v[0]);
+	    }
 	}
 
 	prev_pt_v = pt_v;
@@ -2712,6 +2739,8 @@ GfxCore::SkinPassage(const vector<XSect> & centreline)
 	U[2] = v[2];
 	U[3] = v[3];
 
+	pt_v.set_right_bearing(deg(atan2(right.GetY(), right.GetX())));
+
 	++segment;
     }
 }
@@ -2852,7 +2881,8 @@ GfxCore::OnPrint(const wxString &filename, const wxString &title,
     svxPrintDlg * p;
     p = new svxPrintDlg(m_Parent, filename, title, datestamp,
 			m_PanAngle, m_TiltAngle,
-			m_Names, m_Crosses, m_Legs, m_Surface,
+			m_Names, m_Crosses, m_Legs, m_Surface, m_Tubes,
+			m_Entrances, m_FixedPts, m_ExportedPts,
 			true);
     p->Show(true);
 }
@@ -2860,10 +2890,19 @@ GfxCore::OnPrint(const wxString &filename, const wxString &title,
 void
 GfxCore::OnExport(const wxString &filename, const wxString &title)
 {
+    // Fill in "right_bearing" for each cross-section.
+    list<vector<XSect> >::iterator trav = m_Parent->tubes_begin();
+    list<vector<XSect> >::iterator tend = m_Parent->tubes_end();
+    while (trav != tend) {
+	SkinPassage(*trav, false);
+	++trav;
+    }
+
     svxPrintDlg * p;
     p = new svxPrintDlg(m_Parent, filename, title, wxString(),
 			m_PanAngle, m_TiltAngle,
-			m_Names, m_Crosses, m_Legs, m_Surface,
+			m_Names, m_Crosses, m_Legs, m_Surface, m_Tubes,
+			m_Entrances, m_FixedPts, m_ExportedPts,
 			false);
     p->Show(true);
 }
