@@ -4,7 +4,7 @@
 //  Core drawing code for Aven.
 //
 //  Copyright (C) 2000-2001,2002,2005 Mark R. Shinwell.
-//  Copyright (C) 2001-2004,2005,2006,2007,2010,2011,2012 Olly Betts
+//  Copyright (C) 2001-2004,2005,2006,2007,2010,2011,2012,2013 Olly Betts
 //  Copyright (C) 2005 Martin Green
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -27,7 +27,7 @@
 
 #include <float.h>
 
-#include "img.h"
+#include "img_hosted.h"
 
 #include "guicontrol.h"
 #include "vector3.h"
@@ -101,6 +101,12 @@ enum {
     UPDATE_BLOBS_AND_CROSSES
 };
 
+enum {
+      SPLAYS_HIDE,
+      SPLAYS_SHOW_FADED,
+      SPLAYS_SHOW_NORMAL,
+};
+
 class GfxCore : public GLACanvas {
     Double m_Scale;
     int m_ScaleBarWidth;
@@ -150,6 +156,7 @@ private:
     int m_SwitchingTo;
     bool m_Crosses;
     bool m_Legs;
+    int m_Splays;
     bool m_Names;
     bool m_Scalebar;
     bool m_ColourKey;
@@ -214,7 +221,7 @@ private:
     void DrawTick(int angle_cw);
     void DrawArrow(gla_colour col1, gla_colour col2);
 
-    void SkinPassage(const vector<XSect> & centreline);
+    void SkinPassage(vector<XSect> & centreline, bool draw = true);
 
     virtual void GenerateList(unsigned int l);
     void GenerateDisplayList();
@@ -370,10 +377,12 @@ public:
     int ColouringBy() const { return m_ColourBy; }
 
     bool HasUndergroundLegs() const;
+    bool HasSplays() const;
     bool HasSurfaceLegs() const;
     bool HasTubes() const;
 
     bool ShowingUndergroundLegs() const { return m_Legs; }
+    int ShowingSplaysMode() const { return m_Splays; }
     bool ShowingSurfaceLegs() const { return m_Surface; }
 
     bool ShowingColourKey() const { return m_ColourKey; }
@@ -390,6 +399,12 @@ public:
     void ToggleUndergroundLegs() {
 	ToggleFlag(&m_Legs, UPDATE_BLOBS_AND_CROSSES);
     }
+    void SetSplaysMode(int mode) {
+	m_Splays = mode;
+	UpdateBlobs();
+	InvalidateList(LIST_UNDERGROUND_LEGS);
+	ForceRefresh();
+    }
     void ToggleSurfaceLegs() {
 	ToggleFlag(&m_Surface, UPDATE_BLOBS_AND_CROSSES);
     }
