@@ -4,7 +4,7 @@
 //  OpenGL implementation for the GLA abstraction layer.
 //
 //  Copyright (C) 2002-2003,2005 Mark R. Shinwell
-//  Copyright (C) 2003,2004,2005,2006,2007,2010,2011,2012 Olly Betts
+//  Copyright (C) 2003,2004,2005,2006,2007,2010,2011,2012,2013 Olly Betts
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -35,10 +35,16 @@
 #include "message.h"
 #include "useful.h"
 
+#ifdef HAVE_GL_GL_H
+# include <GL/gl.h>
+#elif defined HAVE_OPENGL_GL_H
+# include <OpenGL/gl.h>
+#endif
+
 #ifdef HAVE_GL_GLEXT_H
 # include <GL/glext.h>
 #elif defined HAVE_OPENGL_GLEXT_H
-#include <OpenGL/glext.h>
+# include <OpenGL/glext.h>
 #endif
 
 #ifndef GL_POINT_SIZE_MAX
@@ -271,10 +277,14 @@ bool GLAList::DrawList() const {
 //  GLACanvas
 //
 
+BEGIN_EVENT_TABLE(GLACanvas, wxGLCanvas)
+    EVT_SIZE(GLACanvas::OnSize)
+END_EVENT_TABLE()
+
 // Pass wxWANTS_CHARS so that the window gets cursor keys on MS Windows.
 GLACanvas::GLACanvas(wxWindow* parent, int id)
-    : wxGLCanvas(parent, id, wxDefaultPosition, wxDefaultSize, wxWANTS_CHARS),
-      m_Translation(), blob_method(UNKNOWN), cross_method(UNKNOWN),
+    : wxGLCanvas(parent, id, NULL, wxDefaultPosition, wxDefaultSize, wxWANTS_CHARS),
+      ctx(this), m_Translation(), blob_method(UNKNOWN), cross_method(UNKNOWN),
       x_size(0), y_size(0)
 {
     // Constructor.
@@ -291,6 +301,7 @@ GLACanvas::GLACanvas(wxWindow* parent, int id)
     m_Fog = false;
     m_AntiAlias = false;
     list_flags = 0;
+    alpha = 1.0;
 }
 
 GLACanvas::~GLACanvas()
@@ -310,7 +321,7 @@ void GLACanvas::FirstShow()
     if (x_size < 1) x_size = 1;
     if (y_size < 1) y_size = 1;
 
-    SetCurrent();
+    ctx.SetCurrent(*this);
     opengl_initialised = true;
 
     // Set the background colour of the canvas to black.
@@ -394,6 +405,7 @@ void GLACanvas::FirstShow()
     path += wxT("unifont.pixelfont");
     if (!m_Font.load(path)) {
 	// FIXME: do something better.
+	// We have this message available: Error in format of font file “%s”
 	fprintf(stderr, "Couldn't load font.\n");
 	exit(1);
     }
@@ -533,8 +545,7 @@ void GLACanvas::OnSize(wxSizeEvent & event)
 	if (y_size < 1) y_size = 1;
     }
 
-    // This apparently is (or at least was) needed for Mac OS X.
-    wxGLCanvas::OnSize(event);
+    event.Skip();
 
     if (!opengl_initialised) return;
 
@@ -575,7 +586,7 @@ void GLACanvas::StartDrawing()
 {
     // Prepare for a redraw operation.
 
-    SetCurrent();
+    ctx.SetCurrent(*this);
     glDepthMask(GL_TRUE);
 
     if (!save_hints) return;
@@ -800,10 +811,10 @@ void GLACanvas::SetDataTransform()
 	glDisable(GL_FOG);
     }
 
+    glEnable(GL_BLEND);
+    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
     if (m_AntiAlias) {
 	glEnable(GL_LINE_SMOOTH);
-	glEnable(GL_BLEND);
-	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
     } else {
 	glDisable(GL_LINE_SMOOTH);
     }
@@ -917,14 +928,14 @@ void GLACanvas::DrawList2D(unsigned int l, glaCoord x, glaCoord y, Double rotati
 void GLACanvas::SetColour(const GLAPen& pen, double rgb_scale)
 {
     // Set the colour for subsequent operations.
-    glColor3f(pen.GetRed() * rgb_scale, pen.GetGreen() * rgb_scale,
-	      pen.GetBlue() * rgb_scale);
+    glColor4f(pen.GetRed() * rgb_scale, pen.GetGreen() * rgb_scale,
+	      pen.GetBlue() * rgb_scale, alpha);
 }
 
 void GLACanvas::SetColour(const GLAPen& pen)
 {
     // Set the colour for subsequent operations.
-    glColor3dv(pen.components);
+    glColor4d(pen.components[0], pen.components[1], pen.components[2], alpha);
 }
 
 void GLACanvas::SetColour(gla_colour colour)
@@ -1459,7 +1470,7 @@ void GLACanvas::ToggleTextured()
     }
 }
 
-bool GLACanvas::SaveScreenshot(const wxString & fnm, int type) const
+bool GLACanvas::SaveScreenshot(const wxString & fnm, wxBitmapType type) const
 {
     const int width = x_size;
     const int height = y_size;
