@@ -4,7 +4,7 @@
 //  Handlers for events relating to the display of a survey.
 //
 //  Copyright (C) 2000-2002,2005 Mark R. Shinwell
-//  Copyright (C) 2001,2003,2004,2005,2006,2011 Olly Betts
+//  Copyright (C) 2001,2003,2004,2005,2006,2011,2012 Olly Betts
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -101,7 +101,7 @@ void GUIControl::HandleScaleRotate(wxPoint point)
 
     int dx, dy;
     int threshold;
-    if (m_ScaleRotateLock == NONE) {
+    if (m_ScaleRotateLock == lock_NONE) {
 	// Dragging to scale or rotate but we've not decided which yet.
 	dx = point.x - m_DragRealStart.x;
 	dy = point.y - m_DragRealStart.y;
@@ -116,30 +116,30 @@ void GUIControl::HandleScaleRotate(wxPoint point)
     if (dx2 + dy2 < threshold) return;
 
     switch (m_ScaleRotateLock) {
-	case NONE:
+	case lock_NONE:
 	    if (dx2 > dy2) {
-		m_ScaleRotateLock = ROTATE;
+		m_ScaleRotateLock = lock_ROTATE;
 //		m_View->SetCursor(GfxCore::CURSOR_ROTATE_HORIZONTALLY);
 	    } else {
-		m_ScaleRotateLock = SCALE;
+		m_ScaleRotateLock = lock_SCALE;
 //		m_View->SetCursor(GfxCore::CURSOR_ZOOM);
 	    }
 	    break;
-	case SCALE:
+	case lock_SCALE:
 	    if (dx2 >= 8 * dy2) {
-		m_ScaleRotateLock = ROTATE;
+		m_ScaleRotateLock = lock_ROTATE;
 //		m_View->SetCursor(GfxCore::CURSOR_ROTATE_HORIZONTALLY);
 	    }
 	    break;
-	case ROTATE:
+	case lock_ROTATE:
 	    if (dy2 >= 8 * dx2) {
-		m_ScaleRotateLock = SCALE;
+		m_ScaleRotateLock = lock_SCALE;
 //		m_View->SetCursor(GfxCore::CURSOR_ZOOM);
 	    }
 	    break;
     }
 
-    if (m_ScaleRotateLock == ROTATE) {
+    if (m_ScaleRotateLock == lock_ROTATE) {
 	dy = 0;
     } else {
 	dx = 0;
@@ -251,10 +251,11 @@ void GUIControl::OnMouseMove(wxMouseEvent& event)
     }
 
     static long timestamp = LONG_MIN;
-    if (dragging != NO_DRAG && m_ScaleRotateLock != NONE && timestamp != LONG_MIN) {
+    if (dragging != NO_DRAG && m_ScaleRotateLock != lock_NONE &&
+	timestamp != LONG_MIN) {
 	// If no motion for a second, reset the direction lock.
 	if (event.GetTimestamp() - timestamp >= 1000) {
-	    m_ScaleRotateLock = NONE;
+	    m_ScaleRotateLock = lock_NONE;
 	    m_DragRealStart = m_DragStart;
 	    RestoreCursor();
 	}
@@ -336,7 +337,7 @@ void GUIControl::OnLButtonDown(wxMouseEvent& event)
 	    }
 
 	    m_LastDrag = drag_MAIN;
-	    m_ScaleRotateLock = NONE;
+	    m_ScaleRotateLock = lock_NONE;
 	}
 
 	m_View->CaptureMouse();
@@ -346,11 +347,12 @@ void GUIControl::OnLButtonDown(wxMouseEvent& event)
 void GUIControl::OnLButtonUp(wxMouseEvent& event)
 {
     if (m_View->HasData()) {
+	if (dragging == NO_DRAG)
+	    return;
+
 	if (event.GetPosition() == m_DragRealStart) {
 	    // Just a "click"...
 	    m_View->CheckHitTestGrid(m_DragStart, true);
-	} else if (dragging == NO_DRAG) {
-	    return;
 	}
 
 //	m_View->RedrawIndicators();
@@ -518,6 +520,44 @@ void GUIControl::OnShowSurveyLegsUpdate(wxUpdateUIEvent& cmd)
     cmd.Check(m_View->ShowingUndergroundLegs());
 }
 
+void GUIControl::OnHideSplays()
+{
+    m_View->SetSplaysMode(SPLAYS_HIDE);
+}
+
+void GUIControl::OnShowSplaysNormal()
+{
+    m_View->SetSplaysMode(SPLAYS_SHOW_NORMAL);
+}
+
+void GUIControl::OnShowSplaysFaded()
+{
+    m_View->SetSplaysMode(SPLAYS_SHOW_FADED);
+}
+
+void GUIControl::OnSplaysUpdate(wxUpdateUIEvent& cmd)
+{
+    cmd.Enable(m_View->HasData() && m_View->HasSplays());
+}
+
+void GUIControl::OnHideSplaysUpdate(wxUpdateUIEvent& cmd)
+{
+    cmd.Enable(m_View->HasData() && m_View->HasSplays());
+    cmd.Check(m_View->ShowingSplaysMode() == SPLAYS_HIDE);
+}
+
+void GUIControl::OnShowSplaysNormalUpdate(wxUpdateUIEvent& cmd)
+{
+    cmd.Enable(m_View->HasData() && m_View->HasSplays());
+    cmd.Check(m_View->ShowingSplaysMode() == SPLAYS_SHOW_NORMAL);
+}
+
+void GUIControl::OnShowSplaysFadedUpdate(wxUpdateUIEvent& cmd)
+{
+    cmd.Enable(m_View->HasData() && m_View->HasSplays());
+    cmd.Check(m_View->ShowingSplaysMode() == SPLAYS_SHOW_FADED);
+}
+
 void GUIControl::OnMoveEast()
 {
     m_View->TurnCaveTo(90.0);
@@ -1134,16 +1174,6 @@ void GUIControl::OnKeyPress(wxKeyEvent &e)
 	    m_View->InvalidateAllLists();
 	    m_View->ForceRefresh();
 	    break;
-#ifdef __WXMAC__
-	// Hugh St Lawrence reports that F11 enters full screen mode, but that
-	// there's no easy way back out (you have to exit aven and then edit
-	// "user(name)/Library/Preferences/aven Preferences" to remove the
-	// "width=-2" setting.  So we process this key press explicitly to
-	// fix this.  He was using the pre-built Survex 1.2.0 and OS X 10.6.8.
-	case WXK_F11:
-	    OnViewFullScreen();
-	    break;
-#endif
 	default:
 	    e.Skip();
     }
