@@ -39,7 +39,7 @@ class GUIControl {
 
     enum { drag_NONE, drag_MAIN, drag_COMPASS, drag_ELEV, drag_SCALE } m_LastDrag;
 
-    enum { NONE, ROTATE, SCALE } m_ScaleRotateLock;
+    enum { lock_NONE, lock_ROTATE, lock_SCALE } m_ScaleRotateLock;
 
     bool m_ReverseControls;
 
@@ -71,6 +71,9 @@ public:
     void OnShowCrosses();
     void OnShowStationNames();
     void OnShowSurveyLegs();
+    void OnHideSplays();
+    void OnShowSplaysNormal();
+    void OnShowSplaysFaded();
     void OnShowSurface();
     void OnMoveEast();
     void OnMoveNorth();
@@ -117,6 +120,10 @@ public:
     void OnShowCrossesUpdate(wxUpdateUIEvent&);
     void OnShowStationNamesUpdate(wxUpdateUIEvent&);
     void OnShowSurveyLegsUpdate(wxUpdateUIEvent&);
+    void OnSplaysUpdate(wxUpdateUIEvent&);
+    void OnHideSplaysUpdate(wxUpdateUIEvent&);
+    void OnShowSplaysNormalUpdate(wxUpdateUIEvent&);
+    void OnShowSplaysFadedUpdate(wxUpdateUIEvent&);
     void OnShowSurfaceUpdate(wxUpdateUIEvent&);
     void OnMoveEastUpdate(wxUpdateUIEvent&);
     void OnMoveNorthUpdate(wxUpdateUIEvent&);
