@@ -4,7 +4,7 @@
 //  Main frame handling for Aven.
 //
 //  Copyright (C) 2000-2002,2005,2006 Mark R. Shinwell
-//  Copyright (C) 2001-2003,2004,2005,2006,2010,2011,2012 Olly Betts
+//  Copyright (C) 2001-2003,2004,2005,2006,2010,2011,2012,2013 Olly Betts
 //  Copyright (C) 2005 Martin Green
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -32,7 +32,7 @@
 #include "aboutdlg.h"
 
 #include "message.h"
-#include "img.h"
+#include "img_hosted.h"
 #include "namecompare.h"
 #include "printwx.h"
 #include "filename.h"
@@ -45,13 +45,26 @@
 #include <wx/process.h>
 #include <wx/regex.h>
 
+#include <cstdlib>
 #include <float.h>
 #include <functional>
+#include <map>
 #include <stack>
 #include <vector>
 
 using namespace std;
 
+const static int img2aven_tab[] = {
+#include "img2aven.h"
+};
+
+inline int
+img2aven(int flags)
+{
+    flags &= (sizeof(img2aven_tab) / sizeof(img2aven_tab[0]) - 1);
+    return img2aven_tab[flags];
+}
+
 class AvenSplitterWindow : public wxSplitterWindow {
     MainFrm *parent;
 
@@ -63,7 +76,7 @@ class AvenSplitterWindow : public wxSplitterWindow {
 	{
 	}
 
-	void OnSplitterDClick(wxSplitterEvent &e) {
+	void OnSplitterDClick(wxSplitterEvent &) {
 	    parent->ToggleSidePanel();
 	}
 
@@ -207,7 +220,7 @@ class AvenPresList : public wxListCtrl {
 	    SetItemCount(entries.size());
 	    modified = true;
 	}
-	void OnDeleteAllItems(wxListEvent& event) {
+	void OnDeleteAllItems(wxListEvent&) {
 	    entries.clear();
 	    SetItemCount(entries.size());
 	    filename = wxString();
@@ -242,6 +255,10 @@ class AvenPresList : public wxListCtrl {
 	}
 	void OnRightClick(wxListEvent& event) {
 	    long item = event.GetIndex();
+	    if (item < 0) {
+		AddMark(item, gfx->GetView());
+		item = 0;
+	    }
 	    EditMarkDlg edit(mainfrm, entries[item]);
 	    if (edit.ShowModal() == wxID_OK) {
 		entries[item] = edit.GetMark();
@@ -451,6 +468,7 @@ BEGIN_EVENT_TABLE(MainFrm, wxFrame)
     EVT_MENU(wxID_FIND, MainFrm::OnGotoFound)
     EVT_MENU(button_HIDE, MainFrm::OnHide)
     EVT_UPDATE_UI(button_HIDE, MainFrm::OnHideUpdate)
+    EVT_IDLE(MainFrm::OnIdle)
 
     EVT_MENU(wxID_OPEN, MainFrm::OnOpen)
     EVT_MENU(wxID_PRINT, MainFrm::OnPrint)
@@ -516,6 +534,9 @@ BEGIN_EVENT_TABLE(MainFrm, wxFrame)
     EVT_MENU(wxID_ZOOM_OUT, MainFrm::OnZoomOut)
     EVT_MENU(menu_ORIENT_DEFAULTS, MainFrm::OnDefaults)
     EVT_MENU(menu_VIEW_SHOW_LEGS, MainFrm::OnShowSurveyLegs)
+    EVT_MENU(menu_SPLAYS_HIDE, MainFrm::OnHideSplays)
+    EVT_MENU(menu_SPLAYS_SHOW_NORMAL, MainFrm::OnShowSplaysNormal)
+    EVT_MENU(menu_SPLAYS_SHOW_FADED, MainFrm::OnShowSplaysFaded)
     EVT_MENU(menu_VIEW_SHOW_CROSSES, MainFrm::OnShowCrosses)
     EVT_MENU(menu_VIEW_SHOW_ENTRANCES, MainFrm::OnShowEntrances)
     EVT_MENU(menu_VIEW_SHOW_FIXED_PTS, MainFrm::OnShowFixedPts)
@@ -571,6 +592,10 @@ BEGIN_EVENT_TABLE(MainFrm, wxFrame)
     EVT_UPDATE_UI(wxID_ZOOM_OUT, MainFrm::OnZoomOutUpdate)
     EVT_UPDATE_UI(menu_ORIENT_DEFAULTS, MainFrm::OnDefaultsUpdate)
     EVT_UPDATE_UI(menu_VIEW_SHOW_LEGS, MainFrm::OnShowSurveyLegsUpdate)
+    EVT_UPDATE_UI(menu_VIEW_SPLAYS, MainFrm::OnSplaysUpdate)
+    EVT_UPDATE_UI(menu_SPLAYS_HIDE, MainFrm::OnHideSplaysUpdate)
+    EVT_UPDATE_UI(menu_SPLAYS_SHOW_NORMAL, MainFrm::OnShowSplaysNormalUpdate)
+    EVT_UPDATE_UI(menu_SPLAYS_SHOW_FADED, MainFrm::OnShowSplaysFadedUpdate)
     EVT_UPDATE_UI(menu_VIEW_SHOW_CROSSES, MainFrm::OnShowCrossesUpdate)
     EVT_UPDATE_UI(menu_VIEW_SHOW_ENTRANCES, MainFrm::OnShowEntrancesUpdate)
     EVT_UPDATE_UI(menu_VIEW_SHOW_FIXED_PTS, MainFrm::OnShowFixedPtsUpdate)
@@ -663,8 +688,10 @@ DnDFile::OnDropFiles(wxCoord, wxCoord, const wxArrayString &filenames)
 MainFrm::MainFrm(const wxString& title, const wxPoint& pos, const wxSize& size) :
     wxFrame(NULL, 101, title, pos, size, wxDEFAULT_FRAME_STYLE),
     m_Gfx(NULL), m_NumEntrances(0), m_NumFixedPts(0), m_NumExportedPts(0),
-    m_NumHighlighted(0), m_HasUndergroundLegs(false), m_HasSurfaceLegs(false),
-    m_HasErrorInformation(false), m_IsExtendedElevation(false)
+    m_NumHighlighted(0),
+    m_HasUndergroundLegs(false), m_HasSplays(false), m_HasSurfaceLegs(false),
+    m_HasErrorInformation(false), m_IsExtendedElevation(false),
+    pending_find(false)
 #ifdef PREFDLG
     , m_PrefsDlg(NULL)
 #endif
@@ -768,7 +795,7 @@ void MainFrm::CreateMenuBar()
     presmenu->Append(menu_PRES_SAVE_AS, wmsg(/*Sa&ve Presentation As…*/314));
     presmenu->AppendSeparator();
     presmenu->Append(menu_PRES_MARK, wmsg(/*&Mark*/315));
-    presmenu->Append(menu_PRES_PLAY, wmsg(/*Pla&y*/316));
+    presmenu->AppendCheckItem(menu_PRES_PLAY, wmsg(/*Pla&y*/316));
     presmenu->Append(menu_PRES_EXPORT_MOVIE, wmsg(/*&Export as Movie…*/317));
 
     wxMenu* viewmenu = new wxMenu;
@@ -781,6 +808,13 @@ void MainFrm::CreateMenuBar()
     viewmenu->AppendSeparator();
     viewmenu->AppendCheckItem(menu_VIEW_SHOW_LEGS, wmsg(/*&Underground Survey Legs\tCtrl+L*/272));
     viewmenu->AppendCheckItem(menu_VIEW_SHOW_SURFACE, wmsg(/*&Surface Survey Legs\tCtrl+F*/291));
+
+    wxMenu* splaymenu = new wxMenu;
+    splaymenu->AppendCheckItem(menu_SPLAYS_HIDE, wmsg(/*&Hide*/407));
+    splaymenu->AppendCheckItem(menu_SPLAYS_SHOW_FADED, wmsg(/*&Fade*/408));
+    splaymenu->AppendCheckItem(menu_SPLAYS_SHOW_NORMAL, wmsg(/*&Show*/409));
+    viewmenu->Append(menu_VIEW_SPLAYS, wmsg(/*Spla&y Legs*/406), splaymenu);
+
     viewmenu->AppendSeparator();
     viewmenu->AppendCheckItem(menu_VIEW_SHOW_OVERLAPPING_NAMES, wmsg(/*&Overlapping Names*/273));
     viewmenu->AppendCheckItem(menu_VIEW_COLOUR_BY_DEPTH, wmsg(/*Colour by &Depth*/292));
@@ -800,7 +834,16 @@ void MainFrm::CreateMenuBar()
     viewmenu->AppendCheckItem(menu_VIEW_FOG, wmsg(/*Fade Distant Ob&jects*/239));
     viewmenu->AppendCheckItem(menu_VIEW_SMOOTH_LINES, wmsg(/*Smoot&hed Survey Legs*/298));
     viewmenu->AppendSeparator();
+#ifdef __WXMAC__
+    // F11 on OS X is used by the desktop (for speaker volume and/or window
+    // navigation).  The standard OS X shortcut for full screen mode is
+    // Shift-Command-F which in wxWidgets terms is Shift+Ctrl+F.
+    wxString wxmac_fullscreen = wmsg(/*Full Screen &Mode\tF11*/356);
+    wxmac_fullscreen.Replace(wxT("\tF11"), wxT("\rShift+Ctrl+F"), false);
+    viewmenu->AppendCheckItem(menu_VIEW_FULLSCREEN, wxmac_fullscreen);
+#else
     viewmenu->AppendCheckItem(menu_VIEW_FULLSCREEN, wmsg(/*Full Screen &Mode\tF11*/356));
+#endif
 #ifdef PREFDLG
     viewmenu->AppendSeparator();
     viewmenu-> Append(wxID_PREFERENCES, wmsg(/*&Preferences…*/347));
@@ -889,10 +932,8 @@ void MainFrm::CreateToolBar()
     m_FindBox = new wxTextCtrl(toolbar, textctrl_FIND, wxString(), wxDefaultPosition,
 			       wxDefaultSize, wxTE_PROCESS_ENTER);
     toolbar->AddControl(m_FindBox);
-    toolbar->AddTool(wxID_FIND, TOOLBAR_BITMAP("find"),
-		     wmsg(/*Find*/332)/*"Search for station name"*/);
-    toolbar->AddTool(button_HIDE, TOOLBAR_BITMAP("hideresults"),
-		     wmsg(/*Hide*/333)/*"Hide search results"*/);
+    toolbar->AddTool(wxID_FIND, wmsg(/*Find*/332), TOOLBAR_BITMAP("find")/*, "Search for station name"*/);
+    toolbar->AddTool(button_HIDE, wmsg(/*Hide*/333), TOOLBAR_BITMAP("hideresults")/*, "Hide search results"*/);
 
     toolbar->Realize();
 }
@@ -962,7 +1003,7 @@ bool MainFrm::LoadData(const wxString& file, wxString prefix)
     // Load the processed survey data.
     img* survey = img_open_survey(file.mb_str(), prefix.mb_str());
     if (!survey) {
-	wxString m = wxString::Format(wmsg(img_error()), file.c_str());
+	wxString m = wxString::Format(wmsg(img_error2msg(img_error())), file.c_str());
 	wxGetApp().ReportError(m);
 	return false;
     }
@@ -978,6 +1019,7 @@ bool MainFrm::LoadData(const wxString& file, wxString prefix)
     m_NumExportedPts = 0;
     m_NumEntrances = 0;
     m_HasUndergroundLegs = false;
+    m_HasSplays = false;
     m_HasSurfaceLegs = false;
     m_HasErrorInformation = false;
 
@@ -1011,9 +1053,13 @@ bool MainFrm::LoadData(const wxString& file, wxString prefix)
     traverse * current_surface_traverse = NULL;
     vector<XSect> * current_tube = NULL;
 
+    map<wxString, LabelInfo *> labelmap;
+    list<LabelInfo*>::const_iterator last_mapped_label = m_Labels.begin();
+
     int result;
     img_point prev_pt = {0,0,0};
     bool current_polyline_is_surface = false;
+    bool current_polyline_is_splay = false;
     bool pending_move = false;
     // When a traverse is split between surface and underground, we split it
     // into contiguous traverses of each, but we need to track these so we can
@@ -1058,16 +1104,19 @@ bool MainFrm::LoadData(const wxString& file, wxString prefix)
 		    complete_dateinfo = false;
 		}
 
+		if (survey->flags & img_FLAG_SPLAY)
+		    m_HasSplays = true;
 		bool is_surface = (survey->flags & img_FLAG_SURFACE);
+		bool is_splay = (survey->flags & img_FLAG_SPLAY);
 		if (!is_surface) {
 		    if (pt.z < m_DepthMin) m_DepthMin = pt.z;
 		    if (pt.z > depthmax) depthmax = pt.z;
 		}
-		if (pending_move || current_polyline_is_surface != is_surface) {
+		if (pending_move || current_polyline_is_surface != is_surface || current_polyline_is_splay != is_splay) {
 		    if (!current_polyline_is_surface && current_traverse) {
 			//FixLRUD(*current_traverse);
 		    }
-		    current_polyline_is_surface = is_surface;
+
 		    // Start new traverse (surface or underground).
 		    if (is_surface) {
 			m_HasSurfaceLegs = true;
@@ -1078,11 +1127,18 @@ bool MainFrm::LoadData(const wxString& file, wxString prefix)
 			m_HasUndergroundLegs = true;
 			traverses.push_back(traverse());
 			current_traverse = &traverses.back();
+			current_traverse->isSplay = is_splay;
 			++n_traverses;
 			// The previous point was at a surface->ug transition.
-			if (prev_pt.z < m_DepthMin) m_DepthMin = prev_pt.z;
-			if (prev_pt.z > depthmax) depthmax = prev_pt.z;
+			if (current_polyline_is_surface) {
+    			    if (prev_pt.z < m_DepthMin) m_DepthMin = prev_pt.z;
+    			    if (prev_pt.z > depthmax) depthmax = prev_pt.z;
+                        }
 		    }
+
+		    current_polyline_is_surface = is_surface;
+		    current_polyline_is_splay = is_splay;
+
 		    if (pending_move) {
 			// Update survey extents.  We only need to do this if
 			// there's a pending move, since for a surface <->
@@ -1115,10 +1171,6 @@ bool MainFrm::LoadData(const wxString& file, wxString prefix)
 	    }
 
 	    case img_LABEL: {
-		int flags = survey->flags;
-		if (flags & img_SFLAG_ENTRANCE) {
-		    flags ^= (img_SFLAG_ENTRANCE | LFLAG_ENTRANCE);
-		}
 		wxString s(survey->label, wxConvUTF8);
 		if (s.empty()) {
 		    // If label isn't valid UTF-8 then this conversion will
@@ -1133,6 +1185,7 @@ bool MainFrm::LoadData(const wxString& file, wxString prefix)
 			s = wxString(survey->label, wxConvISO8859_1);
 		    }
 		}
+		int flags = img2aven(survey->flags);
 		LabelInfo* label = new LabelInfo(pt, s, flags);
 		if (label->IsEntrance()) {
 		    m_NumEntrances++;
@@ -1154,18 +1207,39 @@ bool MainFrm::LoadData(const wxString& file, wxString prefix)
 		    current_tube = &tubes.back();
 		}
 
-		// FIXME: avoid linear search...
-		list<LabelInfo*>::const_iterator i = m_Labels.begin();
+		LabelInfo * lab;
 		wxString label(survey->label, wxConvUTF8);
-		while (i != m_Labels.end() && (*i)->GetText() != label) ++i;
-
-		if (i == m_Labels.end()) {
-		    // Unattached cross-section - ignore for now.
-		    printf("unattached cross-section\n");
-		    if (current_tube->size() <= 1)
-			tubes.resize(tubes.size() - 1);
-		    current_tube = NULL;
-		    break;
+		map<wxString, LabelInfo *>::const_iterator p;
+		p = labelmap.find(label);
+		if (p != labelmap.end()) {
+		    lab = p->second;
+		} else {
+		    // Initialise labelmap lazily - we may have no
+		    // cross-sections.
+		    list<LabelInfo*>::const_iterator i;
+		    if (labelmap.empty()) {
+			i = m_Labels.begin();
+		    } else {
+			i = last_mapped_label;
+			++i;
+		    }
+		    while (i != m_Labels.end() && (*i)->GetText() != label) {
+			labelmap[(*i)->GetText()] = *i;
+			++i;
+		    }
+		    last_mapped_label = i;
+		    if (i == m_Labels.end()) {
+			// Unattached cross-section - ignore for now.
+			printf("unattached cross-section\n");
+			if (current_tube->size() <= 1)
+			    tubes.resize(tubes.size() - 1);
+			current_tube = NULL;
+			if (!m_Labels.empty())
+			    --last_mapped_label;
+			break;
+		    }
+		    lab = *i;
+		    labelmap[label] = lab;
 		}
 
 		int date = survey->days1;
@@ -1175,7 +1249,7 @@ bool MainFrm::LoadData(const wxString& file, wxString prefix)
 		    if (date > datemax) datemax = date;
 		}
 
-		current_tube->push_back(XSect(**i, date, survey->l, survey->r, survey->u, survey->d));
+		current_tube->push_back(XSect(*lab, date, survey->l, survey->r, survey->u, survey->d));
 		break;
 	    }
 
@@ -1231,11 +1305,12 @@ bool MainFrm::LoadData(const wxString& file, wxString prefix)
 		m_NumExportedPts = 0;
 		m_NumEntrances = 0;
 		m_HasUndergroundLegs = false;
+		m_HasSplays = false;
 		m_HasSurfaceLegs = false;
 
 		img_close(survey);
 
-		wxString m = wxString::Format(wmsg(img_error()), file.c_str());
+		wxString m = wxString::Format(wmsg(img_error2msg(img_error())), file.c_str());
 		wxGetApp().ReportError(m);
 
 		return false;
@@ -1259,7 +1334,21 @@ bool MainFrm::LoadData(const wxString& file, wxString prefix)
 
     separator = survey->separator;
     m_Title = wxString(survey->title, wxConvUTF8);
-    m_DateStamp = wxString(survey->datestamp, wxConvUTF8);
+    if (strcmp(survey->datestamp, "?") == 0) {
+	m_DateStamp = wmsg(/*Date and time not available.*/108);
+    } else if (survey->datestamp[0] == '@') {
+	char * end;
+	time_t t = (time_t)strtol(survey->datestamp + 1, &end, 10);
+	if (*end == '\0') {
+	    const struct tm * tm = localtime(&t);
+	    char buf[256];
+	    strftime(buf, 256, msg(/*%a,%Y.%m.%d %H:%M:%S %Z*/107), tm);
+	    m_DateStamp = wxString(buf, wxConvUTF8);
+	}
+    }
+    if (m_DateStamp.empty()) {
+	m_DateStamp = wxString(survey->datestamp, wxConvUTF8);
+    }
     img_close(survey);
 
     // Check we've actually loaded some legs or stations!
@@ -1393,6 +1482,8 @@ void MainFrm::FillTree()
     while (pos != m_Labels.end()) {
 	LabelInfo* label = *pos++;
 
+	if (label->IsAnon()) continue;
+
 	// Determine the current prefix.
 	wxString prefix = label->GetText().BeforeLast(separator);
 
@@ -1655,8 +1746,7 @@ void MainFrm::InitialiseAfterLoad(const wxString & file)
 
     if (!m_FindBox->GetValue().empty()) {
 	// Highlight any stations matching the current search.
-	wxCommandEvent dummy;
-	OnFind(dummy);
+	DoFind();
     }
 
     wxWindow * win = NULL;
@@ -2281,6 +2371,19 @@ void MainFrm::OnPresExportMovieUpdate(wxUpdateUIEvent& event)
 
 void MainFrm::OnFind(wxCommandEvent&)
 {
+    pending_find = true;
+}
+
+void MainFrm::OnIdle(wxIdleEvent&)
+{
+    if (pending_find) {
+	DoFind();
+    }
+}
+
+void MainFrm::DoFind()
+{
+    pending_find = false;
     wxBusyCursor hourglass;
     // Find stations specified by a string or regular expression pattern.
 
