@@ -4,7 +4,7 @@
 //  Main frame handling for Aven.
 //
 //  Copyright (C) 2000-2003,2005 Mark R. Shinwell
-//  Copyright (C) 2001-2003,2004,2005,2006,2010,2011 Olly Betts
+//  Copyright (C) 2001-2003,2004,2005,2006,2010,2011,2012,2013 Olly Betts
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -34,7 +34,7 @@
 #include "aventreectrl.h"
 #include "gfxcore.h"
 #include "guicontrol.h"
-#include "img.h"
+#include "img_hosted.h"
 #include "message.h"
 #include "vector3.h"
 #include "aven.h"
@@ -90,6 +90,10 @@ enum {
     menu_ORIENT_LOWER_VP,
     menu_ORIENT_DEFAULTS,
     menu_VIEW_SHOW_LEGS,
+    menu_VIEW_SPLAYS,
+    menu_SPLAYS_HIDE,
+    menu_SPLAYS_SHOW_FADED,
+    menu_SPLAYS_SHOW_NORMAL,    
     menu_VIEW_SHOW_CROSSES,
     menu_VIEW_SHOW_NAMES,
     menu_VIEW_SHOW_SURFACE,
@@ -139,24 +143,31 @@ public:
 class XSect : public PointInfo {
     friend class MainFrm;
     Double l, r, u, d;
+    Double right_bearing;
 
 public:
-    XSect() : PointInfo(), l(0), r(0), u(0), d(0) { }
+    XSect() : PointInfo(), l(0), r(0), u(0), d(0), right_bearing(0) { }
     XSect(const Point &p, int date_,
 	  Double l_, Double r_, Double u_, Double d_)
-	: PointInfo(p, date_), l(l_), r(r_), u(u_), d(d_) { }
+	: PointInfo(p, date_), l(l_), r(r_), u(u_), d(d_), right_bearing(0) { }
     Double GetL() const { return l; }
     Double GetR() const { return r; }
     Double GetU() const { return u; }
     Double GetD() const { return d; }
+    Double get_right_bearing() const { return right_bearing; }
+    void set_right_bearing(Double right_bearing_) {
+	right_bearing = right_bearing_;
+    }
 };
 
-#define LFLAG_SURFACE		img_SFLAG_SURFACE
-#define LFLAG_UNDERGROUND	img_SFLAG_UNDERGROUND
-#define LFLAG_EXPORTED		img_SFLAG_EXPORTED
-#define LFLAG_FIXED		img_SFLAG_FIXED
-#define LFLAG_ENTRANCE		0x100
-#define LFLAG_HIGHLIGHTED	0x200
+#define LFLAG_NOT_ANON		0x01
+#define LFLAG_NOT_WALL		0x02
+#define LFLAG_SURFACE		0x04
+#define LFLAG_UNDERGROUND	0x08
+#define LFLAG_EXPORTED		0x10
+#define LFLAG_FIXED		0x20
+#define LFLAG_ENTRANCE		0x40
+#define LFLAG_HIGHLIGHTED	0x80
 
 class LabelPlotCmp;
 class AvenPresList;
@@ -170,7 +181,10 @@ public:
     wxTreeItemId tree_id;
 
     LabelInfo(const img_point &pt, const wxString &text_, int flags_)
-	: Point(pt), text(text_), flags(flags_) { }
+	: Point(pt), text(text_), flags(flags_) {
+	if (text.empty())
+	    flags &= ~LFLAG_NOT_ANON;
+    }
     const wxString & GetText() const { return text; }
     int get_flags() const { return flags; }
     void set_flags(int mask) { flags |= mask; }
@@ -184,11 +198,14 @@ public:
     bool IsUnderground() const { return (flags & LFLAG_UNDERGROUND) != 0; }
     bool IsSurface() const { return (flags & LFLAG_SURFACE) != 0; }
     bool IsHighLighted() const { return (flags & LFLAG_HIGHLIGHTED) != 0; }
+    bool IsAnon() const { return (flags & LFLAG_NOT_ANON) == 0; }
+    bool IsWall() const { return (flags & LFLAG_NOT_WALL) == 0; }
 };
 
 class traverse : public vector<PointInfo> {
   public:
     int n_legs;
+    bool isSplay;
     double length;
     double E, H, V;
 
@@ -213,6 +230,7 @@ class MainFrm : public wxFrame {
     int m_NumExportedPts;
     int m_NumHighlighted;
     bool m_HasUndergroundLegs;
+    bool m_HasSplays;
     bool m_HasSurfaceLegs;
     bool m_HasErrorInformation;
     wxSplitterWindow* m_Splitter;
@@ -224,10 +242,6 @@ class MainFrm : public wxFrame {
     wxString m_File;
     wxString m_Title, m_DateStamp;
     wxChar separator; // character separating survey levels (often '.')
-#ifdef PREFDLG
-    PrefsDlg* m_PrefsDlg;
-#endif
-
     Vector3 m_Offsets;
 
     wxString icon_path;
@@ -236,6 +250,11 @@ class MainFrm : public wxFrame {
     wxString here_text, coords_text, dist_text, distfree_text;
 
     bool m_IsExtendedElevation;
+    bool pending_find;
+
+#ifdef PREFDLG
+    PrefsDlg* m_PrefsDlg;
+#endif
 
     void FillTree();
     bool ProcessSVXFile(const wxString & file);
@@ -275,7 +294,9 @@ public:
     void OnPresExportMovieUpdate(wxUpdateUIEvent& event);
     //void OnFileOpenTerrainUpdate(wxUpdateUIEvent& event);
 
+    void DoFind();
     void OnFind(wxCommandEvent& event);
+    void OnIdle(wxIdleEvent& event);
     void OnGotoFound(wxCommandEvent& event);
     void OnHide(wxCommandEvent& event);
     void OnHideUpdate(wxUpdateUIEvent& ui);
@@ -333,6 +354,10 @@ public:
     void OnShowExportedPtsUpdate(wxUpdateUIEvent& event) { if (m_Control) m_Control->OnShowExportedPtsUpdate(event); }
     void OnShowStationNamesUpdate(wxUpdateUIEvent& event) { if (m_Control) m_Control->OnShowStationNamesUpdate(event); }
     void OnShowSurveyLegsUpdate(wxUpdateUIEvent& event) { if (m_Control) m_Control->OnShowSurveyLegsUpdate(event); }
+    void OnSplaysUpdate(wxUpdateUIEvent& event) { if (m_Control) m_Control->OnSplaysUpdate(event); }
+    void OnHideSplaysUpdate(wxUpdateUIEvent& event) { if (m_Control) m_Control->OnHideSplaysUpdate(event); }
+    void OnShowSplaysNormalUpdate(wxUpdateUIEvent& event) { if (m_Control) m_Control->OnShowSplaysNormalUpdate(event); }
+    void OnShowSplaysFadedUpdate(wxUpdateUIEvent& event) { if (m_Control) m_Control->OnShowSplaysFadedUpdate(event); }
     void OnShowSurfaceUpdate(wxUpdateUIEvent& event) { if (m_Control) m_Control->OnShowSurfaceUpdate(event); }
     void OnMoveEastUpdate(wxUpdateUIEvent& event) { if (m_Control) m_Control->OnMoveEastUpdate(event); }
     void OnMoveNorthUpdate(wxUpdateUIEvent& event) { if (m_Control) m_Control->OnMoveNorthUpdate(event); }
@@ -381,6 +406,9 @@ public:
     void OnShowExportedPts(wxCommandEvent&) { if (m_Control) m_Control->OnShowExportedPts(); }
     void OnShowStationNames(wxCommandEvent&) { if (m_Control) m_Control->OnShowStationNames(); }
     void OnShowSurveyLegs(wxCommandEvent&) { if (m_Control) m_Control->OnShowSurveyLegs(); }
+    void OnHideSplays(wxCommandEvent&) { if (m_Control) m_Control->OnHideSplays(); }
+    void OnShowSplaysNormal(wxCommandEvent&) { if (m_Control) m_Control->OnShowSplaysNormal(); }
+    void OnShowSplaysFaded(wxCommandEvent&) { if (m_Control) m_Control->OnShowSplaysFaded(); }
     void OnShowSurface(wxCommandEvent&) { if (m_Control) m_Control->OnShowSurface(); }
     void OnMoveEast(wxCommandEvent&) { if (m_Control) m_Control->OnMoveEast(); }
     void OnMoveNorth(wxCommandEvent&) { if (m_Control) m_Control->OnMoveNorth(); }
@@ -452,6 +480,7 @@ public:
     int GetNumHighlightedPts() const { return m_NumHighlighted; }
 
     bool HasUndergroundLegs() const { return m_HasUndergroundLegs; }
+    bool HasSplays() const { return m_HasSplays; }
     bool HasSurfaceLegs() const { return m_HasSurfaceLegs; }
     bool HasTubes() const { return !tubes.empty(); }
     bool HasErrorInformation() const { return m_HasErrorInformation; }
@@ -490,6 +519,14 @@ public:
 	return tubes.end();
     }
 
+    list<vector<XSect> >::iterator tubes_begin() {
+	return tubes.begin();
+    }
+
+    list<vector<XSect> >::iterator tubes_end() {
+	return tubes.end();
+    }
+
     list<LabelInfo*>::const_iterator GetLabels() const {
 	return m_Labels.begin();
     }
