@@ -261,7 +261,10 @@ default_charset(void)
    if (p == NULL || p[0] == '\0') {
       p = getenv("LC_CTYPE");
       if (p == NULL || p[0] == '\0') {
-	 p = msg_lang;
+	 p = getenv("LANG");
+	 /* Something (AutoCAD?) on Microsoft Windows sets LANG to a number. */
+	 if (p == NULL || !isalpha((unsigned char)p[0]))
+	    p = msg_lang;
       }
    }
 
@@ -613,12 +616,8 @@ add_unicode(int charset, unsigned char *p, int value)
       *p = ' '; return 1;
     case 161 /* ¡ */:
       *p = '!'; return 1;
-    case 171 /* « */:
-      p[1] = *p = '<'; return 2;
     case 176 /* ° */:
       *p = 'd'; p[1] = 'g'; return 2;
-    case 187 /* » */:
-      p[1] = *p = '>'; return 2;
     case 191 /* ¿ */:
       *p = '?'; return 1;
     case 192 /* À */: case 193 /* Á */: case 194 /* Â */: case 195 /* Ã */:
@@ -712,6 +711,7 @@ add_unicode(int charset, unsigned char *p, int value)
       *p = 'z'; return 1;
     case 0x2019: /* &lsquo; */
       *p = '\''; return 1;
+    case 171: /* « */ case 187: /* » */
     case 0x201c: /* &ldquo; */ case 0x201d: /* &rdquo; */
       *p = '"'; return 1;
     case 0x2026: /* &hellip; */
@@ -736,6 +736,7 @@ static const char *pth_cfg_files = "";
 static int num_msgs = 0;
 static char **msg_array = NULL;
 
+static bool msg_lang_explicit = fFalse;
 const char *msg_lang = NULL;
 const char *msg_lang2 = NULL;
 
@@ -805,8 +806,8 @@ parse_msgs(int n, unsigned char *p, int charset_code) {
 #else
 #define N_DONTEXTRACTMSGS 5
 static unsigned char dontextractmsgs[] =
-   "Can't open message file “%s” using path “%s”\0"/*1000*/
-   "Problem with message file “%s”\0"/*1001*/
+   "Can't open message file \"%s\" using path \"%s\"\0"/*1000*/
+   "Problem with message file \"%s\"\0"/*1001*/
    "I don't understand this message file version\0"/*1002*/
    "Message file truncated?\0"/*1003*/
    "Out of memory (couldn't find %lu bytes).\0"/*1004*/;
@@ -839,13 +840,24 @@ parse_msg_file(int charset_code)
 
    fh = fopenWithPthAndExt(pth_cfg_files, fnm, EXT_SVX_MSG, "rb", NULL);
 
-   if (!fh) {
+   if (!fh && strlen(fnm) > 3 && fnm[2] == '_') {
       /* e.g. if 'en_GB' is unknown, see if we know 'en' */
-      if (strlen(fnm) > 3 && fnm[2] == '_') {
-	 fnm[2] = '\0';
-	 fh = fopenWithPthAndExt(pth_cfg_files, fnm, EXT_SVX_MSG, "rb", NULL);
-	 if (!fh) fnm[2] = '_'; /* for error reporting */
+      fnm[2] = '\0';
+      fh = fopenWithPthAndExt(pth_cfg_files, fnm, EXT_SVX_MSG, "rb", NULL);
+      if (!fh) fnm[2] = '_'; /* for error reporting */
+   }
+
+   if (!fh && !msg_lang_explicit) {
+      /* If msg_lang wasn't specified using environment variable SURVEX_LANG,
+       * then default to 'en' if we don't find messages for language msg_lang.
+       */
+      if (fnm[0] && fnm[1]) {
+	 strcpy(fnm, "en");
+      } else {
+	 osfree(fnm);
+	 fnm = osstrdup("en");
       }
+      fh = fopenWithPthAndExt(pth_cfg_files, fnm, EXT_SVX_MSG, "rb", NULL);
    }
 
    if (!fh) {
@@ -1017,6 +1029,11 @@ macosx_got_msg:
    fprintf(stderr, "msg_lang = %p (= \"%s\")\n", msg_lang, msg_lang?msg_lang:"(null)");
 #endif
 
+   msg_lang_explicit = fTrue;
+   if (!msg_lang || !*msg_lang) {
+      msg_lang_explicit = fFalse;
+      msg_lang = getenv("LC_ALL");
+   }
    if (!msg_lang || !*msg_lang) {
       msg_lang = getenv("LC_MESSAGES");
       if (!msg_lang || !*msg_lang) {
@@ -1118,18 +1135,16 @@ macosx_got_msg:
       *p = '_';
    }
 
-   int def_charset = default_charset();
-
 #ifdef LC_MESSAGES
    /* try to setlocale() appropriately too */
    if (!setlocale(LC_MESSAGES, msg_lang)) {
-      if (!msg_lang2 || !setlocale(LC_MESSAGES, msg_lang2)) {
-	 def_charset = CHARSET_ISO_8859_1;
+      if (msg_lang2) {
+	 (void)setlocale(LC_MESSAGES, msg_lang2);
       }
    }
 #endif
 
-   select_charset(def_charset);
+   select_charset(default_charset());
 
 #ifdef HAVE_SIGNAL
    /* Initialise signal handlers only after the messages have been as we need
@@ -1181,14 +1196,16 @@ msgPerm(int en)
 }
 
 void
-v_report(int severity, const char *fnm, int line, int en, va_list ap)
+v_report(int severity, const char *fnm, int line, int col, int en, va_list ap)
 {
 #ifdef AVEN
+   (void)col;
    aven_v_report(severity, fnm, line, en, ap);
 #else
    if (fnm) {
       fputs(fnm, STDERR);
       if (line) fprintf(STDERR, ":%d", line);
+      if (col > 0) fprintf(STDERR, ":%d", col);
    } else {
       fputs(appname_copy, STDERR);
    }
@@ -1222,7 +1239,7 @@ warning(int en, ...)
 {
    va_list ap;
    va_start(ap, en);
-   v_report(0, NULL, 0, en, ap);
+   v_report(0, NULL, 0, 0, en, ap);
    va_end(ap);
 }
 
@@ -1231,7 +1248,7 @@ error(int en, ...)
 {
    va_list ap;
    va_start(ap, en);
-   v_report(1, NULL, 0, en, ap);
+   v_report(1, NULL, 0, 0, en, ap);
    va_end(ap);
 }
 
@@ -1240,7 +1257,7 @@ fatalerror(int en, ...)
 {
    va_list ap;
    va_start(ap, en);
-   v_report(2, NULL, 0, en, ap);
+   v_report(2, NULL, 0, 0, en, ap);
    va_end(ap);
 }
 
@@ -1249,7 +1266,7 @@ warning_in_file(const char *fnm, int line, int en, ...)
 {
    va_list ap;
    va_start(ap, en);
-   v_report(0, fnm, line, en, ap);
+   v_report(0, fnm, line, 0, en, ap);
    va_end(ap);
 }
 
@@ -1258,7 +1275,7 @@ error_in_file(const char *fnm, int line, int en, ...)
 {
    va_list ap;
    va_start(ap, en);
-   v_report(1, fnm, line, en, ap);
+   v_report(1, fnm, line, 0, en, ap);
    va_end(ap);
 }
 
@@ -1267,7 +1284,7 @@ fatalerror_in_file(const char *fnm, int line, int en, ...)
 {
    va_list ap;
    va_start(ap, en);
-   v_report(2, fnm, line, en, ap);
+   v_report(2, fnm, line, 0, en, ap);
    va_end(ap);
 }
 
