@@ -1,7 +1,7 @@
 /* netskel.c
  * Survex network reduction - remove trailing traverses and concatenate
  * traverses between junctions
- * Copyright (C) 1991-2004,2005,2006,2010,2011 Olly Betts
+ * Copyright (C) 1991-2004,2005,2006,2010,2011,2012,2013 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,6 +36,7 @@
 #include "filename.h"
 #include "message.h"
 #include "filelist.h"
+#include "img_hosted.h"
 #include "netartic.h"
 #include "netbits.h"
 #include "netskel.h"
@@ -173,7 +174,7 @@ remove_trailing_travs(void)
 	 do {
 	    struct Link *leg;
 #if PRINT_NETBITS
-	    print_prefix(stn2->name); printf("<%p>",stn2); printf(szLink);
+	    print_prefix(stn2->name); printf("<%p>",stn2); fputs(szLink, stdout);
 #endif
 	    remove_stn_from_list(&stnlist, stn2);
 	    leg = stn2->leg[i];
@@ -258,7 +259,7 @@ concatenate_trav(node *stn, int i)
       stn = stn2;
 
 #if PRINT_NETBITS
-      printf(szLink); print_prefix(stn->name); printf("<%p>",stn);
+      fputs(szLink, stdout); print_prefix(stn->name); printf("<%p>",stn);
 #endif
 
       /* stop if fixed or 3 or 1 node */
@@ -414,7 +415,7 @@ replace_travs(void)
    if (!pimg) {
       char *fnm = add_ext(fnm_output_base, EXT_SVX_3D);
       filename_register_output(fnm);
-      pimg = img_open_write(fnm, survey_title, fTrue);
+      pimg = img_open_write(fnm, survey_title, 0);
       if (!pimg) fatalerror(img_error(), fnm);
       osfree(fnm);
    }
@@ -449,7 +450,8 @@ replace_travs(void)
 	    } else {
 		pimg->days1 = pimg->days2 = -1;
 	    }
-	    img_write_item(pimg, img_LINE, leg->l.flags,
+	    pimg->style = (leg->l.flags >> FLAGS_STYLE_BIT0) & 0x07;
+	    img_write_item(pimg, img_LINE, leg->l.flags & FLAGS_MASK,
 			   sprint_prefix(stn1->name->up),
 			   POS(stn2, 0), POS(stn2, 1), POS(stn2, 2));
 	    if (!(leg->l.reverse & FLAG_ARTICULATION)) {
@@ -636,7 +638,8 @@ replace_travs(void)
 	    } else {
 		pimg->days1 = pimg->days2 = -1;
 	    }
-	    img_write_item(pimg, img_LINE, leg->l.flags,
+	    pimg->style = (leg->l.flags >> FLAGS_STYLE_BIT0) & 0x07;
+	    img_write_item(pimg, img_LINE, leg->l.flags & FLAGS_MASK,
 			   sprint_prefix(leg_pfx),
 			   POS(stn3, 0), POS(stn3, 1), POS(stn3, 2));
 	 }
@@ -714,10 +717,10 @@ err_stat(int cLegsTrav, double lenTrav,
    if (!fSuppress) {
       double sqrt_eTot = sqrt(eTot);
       fputnl(fhErrStat);
-      fprintf(fhErrStat, msg(/*Original length%7.2fm (%3d legs), moved%7.2fm (%5.2fm/leg). */145),
+      fprintf(fhErrStat, msg(/*Original length %6.2fm (%3d legs), moved %6.2fm (%5.2fm/leg). */145),
 	      lenTrav, cLegsTrav, sqrt_eTot, sqrt_eTot / cLegsTrav);
       if (lenTrav > 0.0)
-	 fprintf(fhErrStat, msg(/*Error%7.2f%%*/146), 100 * sqrt_eTot / lenTrav);
+	 fprintf(fhErrStat, msg(/*Error %6.2f%%*/146), 100 * sqrt_eTot / lenTrav);
       else
 	 fputs(msg(/*Error    N/A*/147), fhErrStat);
       fputnl(fhErrStat);
@@ -807,7 +810,8 @@ replace_trailing_travs(void)
 	    } else {
 		pimg->days1 = pimg->days2 = -1;
 	    }
-	    img_write_item(pimg, img_LINE, leg->l.flags,
+	    pimg->style = (leg->l.flags >> FLAGS_STYLE_BIT0) & 0x07;
+	    img_write_item(pimg, img_LINE, leg->l.flags & FLAGS_MASK,
 			   sprint_prefix(leg_pfx),
 			   POS(stn2, 0), POS(stn2, 1), POS(stn2, 2));
 	 }
@@ -844,7 +848,8 @@ replace_trailing_travs(void)
       } else {
 	  pimg->days1 = pimg->days2 = -1;
       }
-      img_write_item(pimg, img_LINE, p->flags,
+      pimg->style = img_STYLE_NOSURVEY;
+      img_write_item(pimg, img_LINE, (p->flags & FLAGS_MASK),
 		     sprint_prefix(p->fr->name->up),
 		     POS(p->to, 0), POS(p->to, 1), POS(p->to, 2));
       nosurveyhead = p->next;
@@ -855,22 +860,35 @@ replace_trailing_travs(void)
    FOR_EACH_STN(stn1, stnlist) {
       int d;
       SVX_ASSERT(fixed(stn1));
-      /* take care of unused fixed points */
-      if (stn1->name->stn == stn1 && stn1->name->ident) {
+      if (stn1->name->stn == stn1) {
 	 int sf = stn1->name->sflags;
+	 /* take care of unused fixed points */
 	 if (!TSTBIT(sf, SFLAGS_SOLVED)) {
-	    /* Set flag to stop station being rewritten after *solve. */
-	    stn1->name->sflags = sf | BIT(SFLAGS_SOLVED);
-	    sf &= SFLAGS_MASK;
-	    if (stn1->name->max_export) sf |= BIT(SFLAGS_EXPORTED);
-	    img_write_item(pimg, img_LABEL, sf, sprint_prefix(stn1->name),
-			   POS(stn1, 0), POS(stn1, 1), POS(stn1, 2));
+	    const char * label = NULL;
+	    if (TSTBIT(sf, SFLAGS_ANON)) {
+	       label = "";
+	    } else if (stn1->name->ident) {
+	       label = sprint_prefix(stn1->name);
+	    }
+	    if (label) {
+	       /* Set flag to stop station being rewritten after *solve. */
+	       stn1->name->sflags = sf | BIT(SFLAGS_SOLVED);
+	       sf &= SFLAGS_MASK;
+	       if (stn1->name->max_export) sf |= BIT(SFLAGS_EXPORTED);
+	       img_write_item(pimg, img_LABEL, sf, label,
+			      POS(stn1, 0), POS(stn1, 1), POS(stn1, 2));
+	    }
 	 }
       }
       /* update coords of bounding box, ignoring the base positions
        * of points fixed with error estimates and only counting stations
-       * in underground surveys. */
-      if (stn1->name->ident && TSTBIT(stn1->name->sflags, SFLAGS_UNDERGROUND)) {
+       * in underground surveys.
+       *
+       * NB We don't set SFLAGS_UNDERGROUND for the anchor station for
+       * a point fixed with error estimates, so this test will exclude
+       * those too, which is what we want.
+       */
+      if (TSTBIT(stn1->name->sflags, SFLAGS_UNDERGROUND)) {
 	 for (d = 0; d < 3; d++) {
 	    if (POS(stn1, d) < min[d]) {
 	       min[d] = POS(stn1, d);
@@ -884,14 +902,19 @@ replace_trailing_travs(void)
       }
 
       d = stn1->name->shape;
-      if (d < 0) {
-	 /* "*fix STN reference ..." sets order negative to suppress the
-	  * unused fixed point warning */
-	 stn1->name->shape = -1 - d;
-      } else if (d <= 1) {
-	 if (d == 0 ||
-	     (stn1->leg[0] && !stn1->leg[0]->l.to->name->ident)) {
-	    /* Unused fixed points without and with error estimates */
+      if (d <= 1 && !TSTBIT(stn1->name->sflags, SFLAGS_USED)) {
+	 bool unused_fixed_point = fFalse;
+	 if (d == 0) {
+	    /* Unused fixed point without error estimates */
+	    unused_fixed_point = fTrue;
+	 } else if (stn1->leg[0]) {
+	    prefix *pfx = stn1->leg[0]->l.to->name;
+	    if (!pfx->ident && !TSTBIT(pfx->sflags, SFLAGS_ANON)) {
+	       /* Unused fixed point with error estimates */
+	       unused_fixed_point = fTrue;
+	    }
+	 }
+	 if (unused_fixed_point) {
 	    warning_in_file(stn1->name->filename, stn1->name->line,
 		    /*Unused fixed point “%s”*/73, sprint_prefix(stn1->name));
 	 }
@@ -900,7 +923,8 @@ replace_trailing_travs(void)
       /* For stations fixed with error estimates, we need to ignore the leg to
        * the "real" fixed point in the node stats.
        */
-      if (stn1->leg[0] && !stn1->leg[0]->l.to->name->ident)
+      if (stn1->leg[0] && !stn1->leg[0]->l.to->name->ident &&
+	  !TSTBIT(stn1->leg[0]->l.to->name->sflags, SFLAGS_ANON))
 	 stn1->name->shape--;
 
       for (i = 0; i <= 2; i++) {
