@@ -1,6 +1,6 @@
 /* printwx.cc */
 /* wxWidgets specific parts of Survex wxWidgets printing code */
-/* Copyright (C) 1993-2003,2004,2005,2006,2010,2011 Olly Betts
+/* Copyright (C) 1993-2003,2004,2005,2006,2010,2011,2012,2013 Olly Betts
  * Copyright (C) 2001,2004 Philip Underwood
  *
  * This program is free software; you can redistribute it and/or modify
@@ -33,12 +33,14 @@ using namespace std;
 #include <ctype.h>
 #include <float.h>
 #include <limits.h>
+#include <wx/confbase.h>
 #include <wx/filename.h>
 #include <wx/print.h>
 #include <wx/printdlg.h>
 #include <wx/spinctrl.h>
 #include <wx/radiobox.h>
 #include <wx/statbox.h>
+#include <wx/valgen.h>
 
 #include "debug.h" /* for BUG and SVX_ASSERT */
 #include "export.h"
@@ -55,18 +57,79 @@ using namespace std;
 
 enum {
 	svx_EXPORT = 1200,
+	svx_FORMAT,
 	svx_SCALE,
 	svx_BEARING,
 	svx_TILT,
 	svx_LEGS,
 	svx_STATIONS,
 	svx_NAMES,
+	svx_XSECT,
+	svx_WALLS,
+	svx_PASSAGES,
 	svx_BORDERS,
 	svx_BLANKS,
 	svx_INFOBOX,
 	svx_SURFACE,
 	svx_PLAN,
-	svx_ELEV
+	svx_ELEV,
+	svx_ENTS,
+	svx_FIXES,
+	svx_EXPORTS,
+	svx_PROJ,
+	svx_GRID,
+	svx_TEXT_HEIGHT,
+	svx_MARKER_SIZE,
+	svx_CENTRED,
+	svx_FULLCOORDS
+};
+
+class BitValidator : public wxValidator {
+    // Disallow assignment.
+    BitValidator & operator=(const BitValidator&);
+
+  protected:
+    int * val;
+
+    int mask;
+
+  public:
+    BitValidator(int * val_, int mask_)
+	: val(val_), mask(mask_) { }
+
+    BitValidator(const BitValidator &o) : wxValidator() {
+	Copy(o);
+    }
+
+    ~BitValidator() { }
+
+    wxObject *Clone() const { return new BitValidator(val, mask); }
+
+    bool Copy(const BitValidator& o) {
+	wxValidator::Copy(o);
+	val = o.val;
+	mask = o.mask;
+	return true;
+    }
+
+    bool Validate(wxWindow *) { return true; }
+
+    bool TransferToWindow() {
+	if (!m_validatorWindow->IsKindOf(CLASSINFO(wxCheckBox)))
+	    return false;
+	((wxCheckBox*)m_validatorWindow)->SetValue(*val & mask);
+	return true;
+    }
+
+    bool TransferFromWindow() {
+	if (!m_validatorWindow->IsKindOf(CLASSINFO(wxCheckBox)))
+	    return false;
+	if (((wxCheckBox*)m_validatorWindow)->IsChecked())
+	    *val |= mask;
+	else
+	    *val &= ~mask;
+	return true;
+    }
 };
 
 class svxPrintout : public wxPrintout {
@@ -121,6 +184,7 @@ class svxPrintout : public wxPrintout {
 };
 
 BEGIN_EVENT_TABLE(svxPrintDlg, wxDialog)
+    EVT_CHOICE(svx_FORMAT, svxPrintDlg::OnChange)
     EVT_TEXT(svx_SCALE, svxPrintDlg::OnChange)
     EVT_COMBOBOX(svx_SCALE, svxPrintDlg::OnChange)
     EVT_SPINCTRL(svx_BEARING, svxPrintDlg::OnChangeSpin)
@@ -138,6 +202,9 @@ BEGIN_EVENT_TABLE(svxPrintDlg, wxDialog)
     EVT_CHECKBOX(svx_STATIONS, svxPrintDlg::OnChange)
     EVT_CHECKBOX(svx_NAMES, svxPrintDlg::OnChange)
     EVT_CHECKBOX(svx_SURFACE, svxPrintDlg::OnChange)
+    EVT_CHECKBOX(svx_ENTS, svxPrintDlg::OnChange)
+    EVT_CHECKBOX(svx_FIXES, svxPrintDlg::OnChange)
+    EVT_CHECKBOX(svx_EXPORTS, svxPrintDlg::OnChange)
 END_EVENT_TABLE()
 
 static wxString scales[] = {
@@ -156,12 +223,64 @@ static wxString scales[] = {
     wxT("100000")
 };
 
+static wxString formats[] = {
+    wxT("DXF"),
+    wxT("EPS"),
+    wxT("GPX"),
+    wxT("HPGL"),
+    wxT("Plot"),
+    wxT("Skencil"),
+    wxT("SVG")
+};
+
+#if 0
+static wxString projs[] = {
+    /* CUCC Austria: */
+    wxT("+proj=tmerc +lat_0=0 +lon_0=13d20 +k=1 +x_0=0 +y_0=-5200000 +ellps=bessel +towgs84=577.326,90.129,463.919,5.137,1.474,5.297,2.4232"),
+    /* British grid SD (Yorkshire): */
+    wxT("+proj=tmerc +lat_0=49d +lon_0=-2d +k=0.999601 +x_0=100000 +y_0=-500000 +ellps=airy +towgs84=375,-111,431,0,0,0,0"),
+    /* British full grid reference: */
+    wxT("+proj=tmerc +lat_0=49d +lon_0=-2d +k=0.999601 +x_0=400000 +y_0=-100000 +ellps=airy +towgs84=375,-111,431,0,0,0,0")
+};
+#endif
+
+static unsigned format_info[] = {
+    LABELS|LEGS|SURF|STNS|PASG|XSECT|WALLS|MARKER_SIZE|TEXT_HEIGHT|GRID|FULL_COORDS,
+    LABELS|LEGS|SURF|STNS,
+    LABELS|LEGS|SURF|ENTS|FIXES|EXPORTS/*|PROJ*/|EXPORT_3D,
+    LABELS|LEGS|SURF|STNS|CENTRED,
+    LABELS|LEGS|SURF,
+    LABELS|LEGS|SURF|STNS|MARKER_SIZE|GRID|SCALE,
+    LABELS|LEGS|SURF|STNS|PASG|XSECT|WALLS|MARKER_SIZE|TEXT_HEIGHT|SCALE
+};
+
+static const char * extension[] = {
+    ".dxf",
+    ".eps",
+    ".gpx",
+    ".hpgl",
+    ".plt",
+    ".sk",
+    ".svg"
+};
+
+static int msg_filetype[] = {
+    /*DXF files*/411,
+    /*EPS files*/412,
+    /*GPX files*/413,
+    /*HPGL for plotters*/414,
+    /*Compass PLT for use with Carto*/415,
+    /*Skencil files*/416,
+    /*SVG files*/417
+};
+
 // there are three jobs to do here...
 // User <-> wx - this should possibly be done in a separate file
 svxPrintDlg::svxPrintDlg(MainFrm* mainfrm_, const wxString & filename,
 			 const wxString & title, const wxString & datestamp,
 			 double angle, double tilt_angle,
 			 bool labels, bool crosses, bool legs, bool surf,
+			 bool tubes, bool ents, bool fixes, bool exports,
 			 bool printing)
 	: wxDialog(mainfrm_, -1, wxString(printing ? wmsg(/*Print*/399) : wmsg(/*Export*/383))),
 	  m_layout(wxGetApp().GetPageSetupDialogData()),
@@ -171,16 +290,25 @@ svxPrintDlg::svxPrintDlg(MainFrm* mainfrm_, const wxString & filename,
     m_printSize = NULL;
     m_bearing = NULL;
     m_tilt = NULL;
-    m_legs = NULL;
-    m_stations = NULL;
-    m_names = NULL;
-    m_borders = NULL;
-    m_infoBox = NULL;
-    m_surface = NULL;
-    m_layout.Labels = labels;
-    m_layout.Crosses = crosses;
-    m_layout.Shots = legs;
-    m_layout.Surface = surf;
+    m_format = NULL;
+    int show_mask = 0;
+    if (labels)
+	show_mask |= LABELS;
+    if (crosses)
+	show_mask |= STNS;
+    if (legs)
+	show_mask |= LEGS;
+    if (surf)
+	show_mask |= SURF;
+    if (tubes)
+	show_mask |= XSECT|WALLS|PASG;
+    if (ents)
+	show_mask |= ENTS;
+    if (fixes)
+	show_mask |= FIXES;
+    if (exports)
+	show_mask |= EXPORTS;
+    m_layout.show_mask = show_mask;
     m_layout.datestamp = datestamp;
     m_layout.rot = int(angle);
     m_layout.title = title;
@@ -203,30 +331,69 @@ svxPrintDlg::svxPrintDlg(MainFrm* mainfrm_, const wxString & filename,
     /* setup our print dialog*/
     wxBoxSizer* v1 = new wxBoxSizer(wxVERTICAL);
     wxBoxSizer* h1 = new wxBoxSizer(wxHORIZONTAL); // holds controls
-    wxBoxSizer* v2 = new wxStaticBoxSizer(new wxStaticBox(this, -1, wmsg(/*View*/283)), wxVERTICAL);
+    m_viewbox = new wxStaticBoxSizer(new wxStaticBox(this, -1, wmsg(/*View*/283)), wxVERTICAL);
     wxBoxSizer* v3 = new wxStaticBoxSizer(new wxStaticBox(this, -1, wmsg(/*Elements*/256)), wxVERTICAL);
-    wxBoxSizer* h2 = new wxBoxSizer(wxHORIZONTAL); // holds buttons
+#if 0
+    wxBoxSizer* h2 = new wxBoxSizer(wxHORIZONTAL);
+#endif
+    wxBoxSizer* h3 = new wxBoxSizer(wxHORIZONTAL); // holds buttons
+
+    if (!printing) {
+	wxStaticText* label;
+	label = new wxStaticText(this, -1, wxString(wmsg(/*Export format*/410)));
+	m_format = new wxChoice(this, svx_FORMAT, wxDefaultPosition, wxDefaultSize,
+				sizeof(formats) / sizeof(formats[0]), formats);
+	unsigned current_format = 0;
+	wxConfigBase * cfg = wxConfigBase::Get();
+	wxString s;
+	if (cfg->Read(wxT("export_format"), &s, wxString())) {
+	    for (unsigned i = 0; i != sizeof(formats) / sizeof(wxString); ++i) {
+		if (s == formats[i]) {
+		    current_format = i;
+		    break;
+		}
+	    }
+	}
+	m_format->SetSelection(current_format);
+	wxBoxSizer* formatbox = new wxBoxSizer(wxHORIZONTAL);
+	formatbox->Add(label, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
+	formatbox->Add(m_format, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
+
+	v1->Add(formatbox, 0, wxALIGN_LEFT|wxALL, 0);
+    }
+
+    wxStaticText* label;
+    label = new wxStaticText(this, -1, wxString(wmsg(/*Scale*/154)) + wxT(" 1:"));
+    if (scales[0].empty()) {
+	if (printing) {
+	    scales[0].assign(wmsg(/*One page*/258));
+	} else {
+	    scales[0].assign(wxT("1000"));
+	}
+    }
+    m_scale = new wxComboBox(this, svx_SCALE, scales[0], wxDefaultPosition,
+			     wxDefaultSize, sizeof(scales) / sizeof(scales[0]),
+			     scales);
+    m_scalebox = new wxBoxSizer(wxHORIZONTAL);
+    m_scalebox->Add(label, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
+    m_scalebox->Add(m_scale, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
+
+    m_viewbox->Add(m_scalebox, 0, wxALIGN_LEFT|wxALL, 0);
 
     if (printing) {
-	wxStaticText* label;
-	label = new wxStaticText(this, -1, wxString(wmsg(/*Scale*/154)) + wxT(" 1:"));
-	if (scales[0].empty()) scales[0].assign(wmsg(/*One page*/258));
-	m_scale = new wxComboBox(this, svx_SCALE, scales[0], wxDefaultPosition,
-				 wxDefaultSize, sizeof(scales) / sizeof(scales[0]),
-				 scales);
-	wxBoxSizer* scalebox = new wxBoxSizer(wxHORIZONTAL);
-	scalebox->Add(label, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
-	scalebox->Add(m_scale, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
-
-	v2->Add(scalebox, 0, wxALIGN_LEFT|wxALL, 0);
-
 	// Make the dummy string wider than any sane value and use that to
 	// fix the width of the control so the sizers allow space for bigger
 	// page layouts.
 	m_printSize = new wxStaticText(this, -1, wxString::Format(wmsg(/*%d pages (%dx%d)*/257), 9604, 98, 98));
-	v2->Add(m_printSize, 0, wxALIGN_LEFT|wxALL, 5);
+	m_viewbox->Add(m_printSize, 0, wxALIGN_LEFT|wxALL, 5);
     }
 
+    /* FIXME:
+     * svx_GRID, // double - spacing, default: 100m
+     * svx_TEXT_HEIGHT, // default 0.6
+     * svx_MARKER_SIZE // default 0.8
+     */
+
     if (m_layout.view != layout::EXTELEV) {
 	wxFlexGridSizer* anglebox = new wxFlexGridSizer(2);
 	wxStaticText * brg_label, * tilt_label;
@@ -241,7 +408,7 @@ svxPrintDlg::svxPrintDlg(MainFrm* mainfrm_, const wxString & filename,
 	m_tilt->SetRange(-90, 90);
 	anglebox->Add(m_tilt, 0, wxALIGN_CENTER|wxALL, 5);
 
-	v2->Add(anglebox, 0, wxALIGN_LEFT|wxALL, 0);
+	m_viewbox->Add(anglebox, 0, wxALIGN_LEFT|wxALL, 0);
 
 	wxBoxSizer * planelevsizer = new wxBoxSizer(wxHORIZONTAL);
 	planelevsizer->Add(new wxButton(this, svx_PLAN, wmsg(/*P&lan view*/117)),
@@ -249,39 +416,98 @@ svxPrintDlg::svxPrintDlg(MainFrm* mainfrm_, const wxString & filename,
 	planelevsizer->Add(new wxButton(this, svx_ELEV, wmsg(/*&Elevation*/285)),
 			   0, wxALIGN_CENTRE_VERTICAL|wxALL, 5);
 
-	v2->Add(planelevsizer, 0, wxALIGN_LEFT|wxALL, 5);
+	m_viewbox->Add(planelevsizer, 0, wxALIGN_LEFT|wxALL, 5);
     }
 
-    h1->Add(v2, 0, wxALIGN_LEFT|wxALL, 5);
-
-    m_legs = new wxCheckBox(this, svx_LEGS, wmsg(/*Underground Survey Legs*/262));
-    v3->Add(m_legs, 0, wxALIGN_LEFT|wxALL, 2);
-    m_surface = new wxCheckBox(this, svx_SURFACE, wmsg(/*Sur&face Survey Legs*/403));
-    v3->Add(m_surface, 0, wxALIGN_LEFT|wxALL, 2);
-    m_stations = new wxCheckBox(this, svx_STATIONS, wmsg(/*Crosses*/261));
-    v3->Add(m_stations, 0, wxALIGN_LEFT|wxALL, 2);
-    m_names = new wxCheckBox(this, svx_NAMES, wmsg(/*Station Names*/260));
-    v3->Add(m_names, 0, wxALIGN_LEFT|wxALL, 2);
+    v3->Add(new wxCheckBox(this, svx_LEGS, wmsg(/*Underground Survey Legs*/262),
+			   wxDefaultPosition, wxDefaultSize, 0,
+			   BitValidator(&m_layout.show_mask, LEGS)),
+	    0, wxALIGN_LEFT|wxALL, 2);
+    v3->Add(new wxCheckBox(this, svx_SURFACE, wmsg(/*Sur&face Survey Legs*/403),
+			   wxDefaultPosition, wxDefaultSize, 0,
+			   BitValidator(&m_layout.show_mask, SURF)),
+	    0, wxALIGN_LEFT|wxALL, 2);
+    v3->Add(new wxCheckBox(this, svx_STATIONS, wmsg(/*Crosses*/261),
+			   wxDefaultPosition, wxDefaultSize, 0,
+			   BitValidator(&m_layout.show_mask, STNS)),
+	    0, wxALIGN_LEFT|wxALL, 2);
+    v3->Add(new wxCheckBox(this, svx_NAMES, wmsg(/*Station Names*/260),
+			   wxDefaultPosition, wxDefaultSize, 0,
+			   BitValidator(&m_layout.show_mask, LABELS)),
+	    0, wxALIGN_LEFT|wxALL, 2);
+    v3->Add(new wxCheckBox(this, svx_ENTS, wmsg(/*Entrances*/418),
+			   wxDefaultPosition, wxDefaultSize, 0,
+			   BitValidator(&m_layout.show_mask, ENTS)),
+	    0, wxALIGN_LEFT|wxALL, 2);
+    v3->Add(new wxCheckBox(this, svx_FIXES, wmsg(/*Fixed Points*/419),
+			   wxDefaultPosition, wxDefaultSize, 0,
+			   BitValidator(&m_layout.show_mask, FIXES)),
+	    0, wxALIGN_LEFT|wxALL, 2);
+    v3->Add(new wxCheckBox(this, svx_EXPORTS, wmsg(/*Entrances*/420),
+			   wxDefaultPosition, wxDefaultSize, 0,
+			   BitValidator(&m_layout.show_mask, EXPORTS)),
+	    0, wxALIGN_LEFT|wxALL, 2);
+    v3->Add(new wxCheckBox(this, svx_XSECT, wmsg(/*Cross-sections*/393),
+			   wxDefaultPosition, wxDefaultSize, 0,
+			   BitValidator(&m_layout.show_mask, XSECT)),
+	    0, wxALIGN_LEFT|wxALL, 2);
+    if (!printing) {
+	v3->Add(new wxCheckBox(this, svx_WALLS, wmsg(/*Walls*/394),
+			       wxDefaultPosition, wxDefaultSize, 0,
+			       BitValidator(&m_layout.show_mask, WALLS)),
+		0, wxALIGN_LEFT|wxALL, 2);
+	v3->Add(new wxCheckBox(this, svx_PASSAGES, wmsg(/*Passages*/395),
+			       wxDefaultPosition, wxDefaultSize, 0,
+			       BitValidator(&m_layout.show_mask, PASG)),
+		0, wxALIGN_LEFT|wxALL, 2);
+	v3->Add(new wxCheckBox(this, svx_CENTRED, wmsg(/*Origin in centre*/421),
+			       wxDefaultPosition, wxDefaultSize, 0,
+			       BitValidator(&m_layout.show_mask, CENTRED)),
+		0, wxALIGN_LEFT|wxALL, 2);
+	v3->Add(new wxCheckBox(this, svx_FULLCOORDS, wmsg(/*Full coordinates*/422),
+			       wxDefaultPosition, wxDefaultSize, 0,
+			       BitValidator(&m_layout.show_mask, FULL_COORDS)),
+		0, wxALIGN_LEFT|wxALL, 2);
+    }
     if (printing) {
-	m_borders = new wxCheckBox(this, svx_BORDERS, wmsg(/*Page Borders*/264));
-	v3->Add(m_borders, 0, wxALIGN_LEFT|wxALL, 2);
+	v3->Add(new wxCheckBox(this, svx_BORDERS, wmsg(/*Page Borders*/264),
+			       wxDefaultPosition, wxDefaultSize, 0,
+			       wxGenericValidator(&m_layout.Border)),
+		0, wxALIGN_LEFT|wxALL, 2);
 //	m_blanks = new wxCheckBox(this, svx_BLANKS, wmsg(/*Blank Pages*/266));
 //	v3->Add(m_blanks, 0, wxALIGN_LEFT|wxALL, 2);
-	m_infoBox = new wxCheckBox(this, svx_INFOBOX, wmsg(/*Info Box*/265));
-	v3->Add(m_infoBox, 0, wxALIGN_LEFT|wxALL, 2);
+	v3->Add(new wxCheckBox(this, svx_INFOBOX, wmsg(/*Info Box*/265),
+			       wxDefaultPosition, wxDefaultSize, 0,
+			       wxGenericValidator(&m_layout.Legend)),
+		0, wxALIGN_LEFT|wxALL, 2);
     }
 
     h1->Add(v3, 0, wxALIGN_LEFT|wxALL, 5);
+    h1->Add(m_viewbox, 0, wxALIGN_LEFT|wxALL, 5);
 
     v1->Add(h1, 0, wxALIGN_LEFT|wxALL, 5);
 
+#if 0 // FIXME: too wide as-is
+    h2->Add(new wxStaticText(this, -1, wmsg(/*input datum as string to pass to PROJ*/389)));
+    h2->Add(new wxComboBox(this, svx_PROJ, projs[0], wxDefaultPosition,
+			   wxDefaultSize, sizeof(projs) / sizeof(projs[0]),
+			   projs));
+    v1->Add(h2, 0, wxALIGN_LEFT, 5);
+#endif
+
+    // When we enable/disable checkboxes in the export dialog, ideally we'd
+    // like the dialog to resize, but not sure how to achieve that, so we
+    // add a stretchable spacer here so at least the buttons stay in the
+    // lower right corner.
+    v1->AddStretchSpacer();
+
     wxButton * but;
     but = new wxButton(this, wxID_CANCEL);
-    h2->Add(but, 0, wxALIGN_RIGHT|wxALL, 5);
+    h3->Add(but, 0, wxALIGN_RIGHT|wxALL, 5);
     if (printing) {
 #ifdef AVEN_PRINT_PREVIEW
 	but = new wxButton(this, wxID_PREVIEW);
-	h2->Add(but, 0, wxALIGN_RIGHT|wxALL, 5);
+	h3->Add(but, 0, wxALIGN_RIGHT|wxALL, 5);
 	but = new wxButton(this, wxID_PRINT);
 #else
 	but = new wxButton(this, wxID_PRINT, wmsg(/*&Print…*/400));
@@ -290,8 +516,8 @@ svxPrintDlg::svxPrintDlg(MainFrm* mainfrm_, const wxString & filename,
 	but = new wxButton(this, svx_EXPORT, wmsg(/*&Export…*/230));
     }
     but->SetDefault();
-    h2->Add(but, 0, wxALIGN_RIGHT|wxALL, 5);
-    v1->Add(h2, 0, wxALIGN_RIGHT|wxALL, 5);
+    h3->Add(but, 0, wxALIGN_RIGHT|wxALL, 5);
+    v1->Add(h3, 0, wxALIGN_RIGHT|wxALL, 5);
 
     SetAutoLayout(true);
     SetSizer(v1);
@@ -299,12 +525,13 @@ svxPrintDlg::svxPrintDlg(MainFrm* mainfrm_, const wxString & filename,
     v1->SetSizeHints(this);
 
     LayoutToUI();
-    SomethingChanged();
+    SomethingChanged(0);
 }
 
 void
 svxPrintDlg::OnPrint(wxCommandEvent&) {
-    SomethingChanged();
+    SomethingChanged(0);
+    TransferDataFromWindow();
     wxPageSetupDialogData * psdd = wxGetApp().GetPageSetupDialogData();
     wxPrintDialogData pd(psdd->GetPrintData());
     wxPrinter pr(&pd);
@@ -318,16 +545,39 @@ svxPrintDlg::OnPrint(wxCommandEvent&) {
 void
 svxPrintDlg::OnExport(wxCommandEvent&) {
     UIToLayout();
-    wxString baseleaf;
-    wxFileName::SplitPath(m_File, NULL, NULL, &baseleaf, NULL, wxPATH_NATIVE);
-    wxFileDialog dlg(this, wmsg(/*Export as:*/401), wxString(), baseleaf,
-		     wmsg(/*DXF files|*.dxf|SVG files|*.svg|Sketch files|*.sk|EPS files|*.eps|Compass PLT for use with Carto|*.plt|HPGL for plotters|*.hpgl*/96),
-		     wxFD_SAVE|wxFD_OVERWRITE_PROMPT);
+    TransferDataFromWindow();
+    wxString leaf;
+    wxFileName::SplitPath(m_File, NULL, NULL, &leaf, NULL, wxPATH_NATIVE);
+    unsigned format_idx = ((wxChoice*)FindWindow(svx_FORMAT))->GetSelection();
+    leaf += wxString::FromUTF8(extension[format_idx]);
+
+    wxString filespec = wmsg(msg_filetype[format_idx]);
+    filespec += wxT("|*");
+    filespec += wxString::FromUTF8(extension[format_idx]);
+    filespec += wxT("|");
+    filespec += wmsg(/*All files*/208);
+    filespec += wxT("|");
+    filespec += wxFileSelectorDefaultWildcardStr;
+
+    wxFileDialog dlg(this, wmsg(/*Export as:*/401), wxString(), leaf,
+		     filespec, wxFD_SAVE|wxFD_OVERWRITE_PROMPT);
     if (dlg.ShowModal() == wxID_OK) {
+#if 0 // FIXME: sort this out
+	wxString input_projection = ((wxComboBox*)FindWindow(svx_PROJ))->GetValue();
+#else
+	wxConfigBase * cfg = wxConfigBase::Get();
+	wxString input_projection;
+	cfg->Read(wxT("input_projection"), &input_projection,
+		  wxString(wxT("+proj=tmerc +lat_0=0 +lon_0=13d20 +k=1 +x_0=0 +y_0=-5200000 +ellps=bessel +towgs84=577.326,90.129,463.919,5.137,1.474,5.297,2.4232")));
+#endif
+	double grid = 100; // metres
+	double text_height = 0.6;
+	double marker_size = 0.8;
+
 	if (!Export(dlg.GetPath(), m_layout.title, mainfrm,
-		    m_layout.rot, m_layout.tilt,
-		    m_layout.Labels, m_layout.Crosses,
-		    m_layout.Shots, m_layout.Surface)) {
+		    m_layout.rot, m_layout.tilt, m_layout.show_mask,
+		    export_format(format_idx), input_projection.mb_str(),
+		    grid, text_height, marker_size)) {
 	    wxString m = wxString::Format(wmsg(/*Couldn’t write file “%s”*/402).c_str(),
 					  m_File.c_str());
 	    wxGetApp().ReportError(m);
@@ -339,7 +589,8 @@ svxPrintDlg::OnExport(wxCommandEvent&) {
 #ifdef AVEN_PRINT_PREVIEW
 void
 svxPrintDlg::OnPreview(wxCommandEvent&) {
-    SomethingChanged();
+    SomethingChanged(0);
+    TransferDataFromWindow();
     wxPageSetupDialogData * psdd = wxGetApp().GetPageSetupDialogData();
     wxPrintDialogData pd(psdd->GetPrintData());
     wxPrintPreview* pv;
@@ -385,13 +636,13 @@ svxPrintDlg::OnPreview(wxCommandEvent&) {
 void
 svxPrintDlg::OnPlan(wxCommandEvent&) {
     m_tilt->SetValue(-90);
-    SomethingChanged();
+    SomethingChanged(svx_TILT);
 }
 
 void
 svxPrintDlg::OnElevation(wxCommandEvent&) {
     m_tilt->SetValue(0);
-    SomethingChanged();
+    SomethingChanged(svx_TILT);
 }
 
 void
@@ -405,17 +656,47 @@ svxPrintDlg::OnElevationUpdate(wxUpdateUIEvent& e) {
 }
 
 void
-svxPrintDlg::OnChangeSpin(wxSpinEvent&e) {
-    SomethingChanged();
+svxPrintDlg::OnChangeSpin(wxSpinEvent& e) {
+    SomethingChanged(e.GetId());
 }
 
 void
-svxPrintDlg::OnChange(wxCommandEvent&) {
-    SomethingChanged();
+svxPrintDlg::OnChange(wxCommandEvent& e) {
+    SomethingChanged(e.GetId());
 }
 
 void
-svxPrintDlg::SomethingChanged() {
+svxPrintDlg::SomethingChanged(int control_id) {
+    if ((control_id == 0 || control_id == svx_FORMAT) && m_format) {
+	// Update the shown/hidden fields for the newly selected export filter.
+	int new_filter_idx = m_format->GetSelection();
+	if (new_filter_idx != wxNOT_FOUND) {
+	    unsigned mask = format_info[new_filter_idx];
+	    FindWindow(svx_LEGS)->Show(mask & LEGS);
+	    FindWindow(svx_SURFACE)->Show(mask & SURF);
+	    FindWindow(svx_STATIONS)->Show(mask & STNS);
+	    FindWindow(svx_NAMES)->Show(mask & LABELS);
+	    FindWindow(svx_XSECT)->Show(mask & XSECT);
+	    FindWindow(svx_WALLS)->Show(mask & WALLS);
+	    FindWindow(svx_PASSAGES)->Show(mask & PASG);
+	    FindWindow(svx_ENTS)->Show(mask & ENTS);
+	    FindWindow(svx_FIXES)->Show(mask & FIXES);
+	    FindWindow(svx_EXPORTS)->Show(mask & EXPORTS);
+	    FindWindow(svx_CENTRED)->Show(mask & CENTRED);
+	    FindWindow(svx_FULLCOORDS)->Show(mask & FULL_COORDS);
+#if 0
+	    FindWindow(svx_PROJ)->Show(mask & PROJ);
+#endif
+	    m_scalebox->Show(bool(mask & SCALE));
+	    m_viewbox->Show(!bool(mask & EXPORT_3D));
+	    GetSizer()->Layout();
+	    if (control_id == svx_FORMAT) {
+		wxConfigBase * cfg = wxConfigBase::Get();
+		cfg->Write(wxT("export_format"), formats[new_filter_idx]);
+	    }
+	}
+    }
+
     UIToLayout();
     if (!m_printSize) return;
     // Update the bounding box.
@@ -436,13 +717,7 @@ svxPrintDlg::SomethingChanged() {
 
 void
 svxPrintDlg::LayoutToUI(){
-    m_names->SetValue(m_layout.Labels);
-    m_legs->SetValue(m_layout.Shots);
-    m_stations->SetValue(m_layout.Crosses);
-    if (m_borders) m_borders->SetValue(m_layout.Border);
 //    m_blanks->SetValue(m_layout.SkipBlank);
-    if (m_infoBox) m_infoBox->SetValue(!m_layout.Raw);
-    m_surface->SetValue(m_layout.Surface);
     if (m_layout.view != layout::EXTELEV) {
 	m_tilt->SetValue(m_layout.tilt);
 	m_bearing->SetValue(m_layout.rot);
@@ -463,13 +738,7 @@ svxPrintDlg::LayoutToUI(){
 
 void
 svxPrintDlg::UIToLayout(){
-    m_layout.Labels = m_names->IsChecked();
-    m_layout.Shots = m_legs->IsChecked();
-    m_layout.Crosses = m_stations->IsChecked();
-    if (m_borders) m_layout.Border = m_borders->IsChecked();
 //    m_layout.SkipBlank = m_blanks->IsChecked();
-    if (m_infoBox) m_layout.Raw = !m_infoBox->IsChecked();
-    m_layout.Surface = m_surface->IsChecked();
 
     if (m_layout.view != layout::EXTELEV) {
 	m_layout.tilt = m_tilt->GetValue();
@@ -480,6 +749,16 @@ svxPrintDlg::UIToLayout(){
 	} else {
 	    m_layout.view = layout::TILT;
 	}
+
+	bool enable_passage_opts = (m_layout.view != layout::TILT);
+	wxWindow * win;
+	win = FindWindow(svx_XSECT);
+	if (win) win->Enable(enable_passage_opts);
+	win = FindWindow(svx_WALLS);
+	if (win) win->Enable(enable_passage_opts);
+	win = FindWindow(svx_PASSAGES);
+	if (win) win->Enable(enable_passage_opts);
+
 	m_layout.rot = m_bearing->GetValue();
     }
 }
@@ -495,7 +774,7 @@ svxPrintDlg::RecalcBounds()
     double SINT = sin(rad(m_layout.tilt));
     double COST = cos(rad(m_layout.tilt));
 
-    if (m_layout.Shots) {
+    if (m_layout.show_mask & LEGS) {
 	list<traverse>::const_iterator trav = mainfrm->traverses_begin();
 	list<traverse>::const_iterator tend = mainfrm->traverses_end();
 	for ( ; trav != tend; ++trav) {
@@ -514,7 +793,7 @@ svxPrintDlg::RecalcBounds()
 	    }
 	}
     }
-    if (m_layout.Surface) {
+    if (m_layout.show_mask & SURF) {
 	list<traverse>::const_iterator trav = mainfrm->surface_traverses_begin();
 	list<traverse>::const_iterator tend = mainfrm->surface_traverses_end();
 	for ( ; trav != tend; ++trav) {
@@ -533,13 +812,13 @@ svxPrintDlg::RecalcBounds()
 	    }
 	}
     }
-    if (m_layout.Labels || m_layout.Crosses) {
+    if (m_layout.show_mask & (LABELS|STNS)) {
 	list<LabelInfo*>::const_iterator label = mainfrm->GetLabels();
 	while (label != mainfrm->GetLabelsEnd()) {
 	    double x = (*label)->GetX();
 	    double y = (*label)->GetY();
 	    double z = (*label)->GetZ();
-	    if (m_layout.Surface || (*label)->IsUnderground()) {
+	    if ((m_layout.show_mask & SURF) || (*label)->IsUnderground()) {
 		double X = x * COS - y * SIN;
 		if (X > m_layout.xMax) m_layout.xMax = X;
 		if (X < m_layout.xMin) m_layout.xMin = X;
@@ -1019,14 +1298,14 @@ svxPrintout::OnPrintPage(int pageNum) {
 
     NewPage(pageNum, l->pagesX, l->pagesY);
 
-    if (!l->Raw && pageNum == (l->pagesY - 1) * l->pagesX + 1) {
+    if (l->Legend && pageNum == (l->pagesY - 1) * l->pagesX + 1) {
 	SetFont(PR_FONT_DEFAULT);
 	draw_info_box();
     }
 
     const double Sc = 1000 / l->Scale;
 
-    if (l->Shots) {
+    if (l->show_mask & LEGS) {
 	SetColour(PR_COLOUR_LEG);
 	list<traverse>::const_iterator trav = mainfrm->traverses_begin();
 	list<traverse>::const_iterator tend = mainfrm->traverses_end();
@@ -1050,7 +1329,8 @@ svxPrintout::OnPrintPage(int pageNum) {
 	}
     }
 
-    if (l->Shots && (l->tilt == 0.0 || l->tilt == 90.0 || l->tilt == -90.0)) {
+    if ((l->show_mask & XSECT) &&
+	(l->tilt == 0.0 || l->tilt == 90.0 || l->tilt == -90.0)) {
 	list<vector<XSect> >::const_iterator trav = mainfrm->tubes_begin();
 	list<vector<XSect> >::const_iterator tend = mainfrm->tubes_end();
 	for ( ; trav != tend; ++trav) {
@@ -1059,7 +1339,7 @@ svxPrintout::OnPrintPage(int pageNum) {
 	}
     }
 
-    if (l->Surface) {
+    if (l->show_mask & SURF) {
 	SetColour(PR_COLOUR_SURFACE_LEG);
 	list<traverse>::const_iterator trav = mainfrm->surface_traverses_begin();
 	list<traverse>::const_iterator tend = mainfrm->surface_traverses_end();
@@ -1083,24 +1363,24 @@ svxPrintout::OnPrintPage(int pageNum) {
 	}
     }
 
-    if (l->Labels || l->Crosses) {
-	if (l->Labels) SetFont(PR_FONT_LABELS);
+    if (l->show_mask & (LABELS|STNS)) {
+	if (l->show_mask & LABELS) SetFont(PR_FONT_LABELS);
 	list<LabelInfo*>::const_iterator label = mainfrm->GetLabels();
 	while (label != mainfrm->GetLabelsEnd()) {
 	    double px = (*label)->GetX();
 	    double py = (*label)->GetY();
 	    double pz = (*label)->GetZ();
-	    if (l->Surface || (*label)->IsUnderground()) {
+	    if ((l->show_mask & SURF) || (*label)->IsUnderground()) {
 		double X = px * COS - py * SIN;
 		double Y = pz * COST - (px * SIN + py * COS) * SINT;
 		long xnew, ynew;
 		xnew = (long)((X * Sc + l->xOrg) * l->scX);
 		ynew = (long)((Y * Sc + l->yOrg) * l->scY);
-		if (l->Crosses) {
+		if (l->show_mask & STNS) {
 		    SetColour(PR_COLOUR_CROSS);
 		    DrawCross(xnew, ynew);
 		}
-		if (l->Labels) {
+		if (l->show_mask & LABELS) {
 		    SetColour(PR_COLOUR_LABELS);
 		    MoveTo(xnew, ynew);
 		    WriteString((*label)->GetText());
